@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2016 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,12 +14,6 @@
  * limitations under the License.
  *
 */
-#ifdef _WIN32
-  // Ensure that Winsock2.h is included before Windows.h, which can get
-  // pulled in by anybody (e.g., Boost).
-  #include <Winsock2.h>
-#endif
-
 #include <boost/bind.hpp>
 
 #include "gazebo/transport/transport.hh"
@@ -123,12 +117,12 @@ ModelRightMenu::ModelRightMenu()
   connect(state->action, SIGNAL(triggered()), state, SLOT(Callback()));
   this->viewStates.push_back(state);
 
-  // \todo Reimplement
-  // this->skeletonAction = new QAction(tr("Skeleton"), this);
-  // this->skeletonAction->setStatusTip(tr("Show model skeleton"));
-  // this->skeletonAction->setCheckable(true);
-  // connect(this->skeletonAction, SIGNAL(triggered()), this,
-  //         SLOT(OnSkeleton()));
+  state = new ViewState(this, "show_skeleton", "hide_skeleton");
+  state->action = new QAction(tr("Skeleton"), this);
+  state->action->setStatusTip(tr("Show model skeleton"));
+  state->action->setCheckable(true);
+  connect(state->action, SIGNAL(triggered()), state, SLOT(Callback()));
+  this->viewStates.push_back(state);
 
   // Window mode
   this->windowMode = "Simulation";
@@ -143,7 +137,7 @@ ModelRightMenu::ModelRightMenu()
 bool ModelRightMenu::Init()
 {
   this->node = transport::NodePtr(new transport::Node());
-  this->node->Init();
+  this->node->TryInit(common::Time::Maximum());
   this->requestSub = this->node->Subscribe("~/request",
       &ModelRightMenu::OnRequest, this);
 
@@ -166,22 +160,17 @@ bool ModelRightMenu::OnKeyRelease(const common::KeyEvent &_event)
 /////////////////////////////////////////////////
 ModelRightMenu::~ModelRightMenu()
 {
+  this->requestSub.reset();
   this->node->Fini();
+  this->node.reset();
 }
 
 /////////////////////////////////////////////////
 void ModelRightMenu::Run(const std::string &_entityName, const QPoint &_pt,
     EntityTypes _type)
 {
-  // Find out the entity type
-  if (_type == EntityTypes::MODEL || _type == EntityTypes::LIGHT)
-  {
-    this->entityName = _entityName.substr(0, _entityName.find("::"));
-  }
-  else if (_type == EntityTypes::LINK)
-  {
-    this->entityName = _entityName;
-  }
+  // Set to scoped name
+  this->entityName = _entityName;
 
   QMenu menu;
 
@@ -247,9 +236,6 @@ void ModelRightMenu::Run(const std::string &_entityName, const QPoint &_pt,
     menu.addAction(g_deleteAct);
   }
 
-  /// \todo Reimplement these features.
-  // menu.addAction(this->skeletonAction);
-
   menu.exec(_pt);
 }
 
@@ -290,16 +276,28 @@ void ModelRightMenu::OnApplyWrench()
   }
 
   std::string modelName, linkName;
-  if (vis == vis->GetRootVisual())
+  if (vis->GetType() == rendering::Visual::VT_MODEL)
   {
     modelName = this->entityName;
     // If model selected just take the first link
-    linkName = vis->GetChild(0)->GetName();
+    for (unsigned int i = 0; i < vis->GetChildCount(); ++i)
+    {
+      rendering::VisualPtr currentChild = vis->GetChild(i);
+      if (currentChild->GetType() == rendering::Visual::VT_LINK)
+      {
+        linkName = currentChild->Name();
+        break;
+      }
+    }
   }
   else
   {
-    modelName = vis->GetRootVisual()->GetName();
-    linkName = this->entityName;
+    // Links should always have a parent
+    if (vis->GetParent() != nullptr)
+    {
+      modelName = vis->GetParent()->Name();
+      linkName = this->entityName;
+    }
   }
 
   applyWrenchDialog->Init(modelName, linkName);
@@ -413,26 +411,3 @@ void ModelRightMenu::OnWindowMode(const std::string &_mode)
 {
   this->windowMode = _mode;
 }
-
-/// \todo Reimplement these functions.
-/////////////////////////////////////////////////
-// void ModelRightMenu::OnSkeleton()
-// {
-//   this->skeletonActionState[this->entityName] =
-//     this->skeletonAction->isChecked();
-//
-//   if (this->skeletonAction->isChecked())
-//   {
-//     this->requestMsg = msgs::CreateRequest("show_skeleton",
-//         this->entityName);
-//     this->requestMsg->set_dbl_data(1.0);
-//   }
-//   else
-//   {
-//     this->requestMsg = msgs::CreateRequest("show_skeleton",
-//         this->entityName);
-//     this->requestMsg->set_dbl_data(0.0);
-//   }
-//
-//   this->requestPub->Publish(*this->requestMsg);
-// }
