@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2016 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,12 +15,6 @@
  *
 */
 
-#ifdef _WIN32
-  // Ensure that Winsock2.h is included before Windows.h, which can get
-  // pulled in by anybody (e.g., Boost).
-  #include <Winsock2.h>
-#endif
-
 #include <tbb/parallel_for.h>
 #include <tbb/blocked_range.h>
 #include <float.h>
@@ -28,16 +22,21 @@
 #include <boost/bind.hpp>
 #include <boost/function.hpp>
 #include <boost/thread/recursive_mutex.hpp>
+#include <ignition/math/Pose3.hh>
+#include <ignition/math/SemanticVersion.hh>
+#include <ignition/msgs/plugin_v.pb.h>
 #include <sstream>
 
-#include "gazebo/util/OpenAL.hh"
 #include "gazebo/common/KeyFrame.hh"
 #include "gazebo/common/Animation.hh"
 #include "gazebo/common/Plugin.hh"
 #include "gazebo/common/Events.hh"
 #include "gazebo/common/Exception.hh"
 #include "gazebo/common/Console.hh"
+#include "gazebo/common/CommonIface.hh"
 #include "gazebo/common/CommonTypes.hh"
+#include "gazebo/common/SdfFrameSemantics.hh"
+#include "gazebo/common/URI.hh"
 
 #include "gazebo/physics/Gripper.hh"
 #include "gazebo/physics/Joint.hh"
@@ -50,6 +49,10 @@
 
 #include "gazebo/transport/Node.hh"
 
+#include "gazebo/util/IgnMsgSdf.hh"
+#include "gazebo/util/IntrospectionManager.hh"
+#include "gazebo/util/OpenAL.hh"
+
 using namespace gazebo;
 using namespace physics;
 
@@ -69,8 +72,63 @@ Model::~Model()
 //////////////////////////////////////////////////
 void Model::Load(sdf::ElementPtr _sdf)
 {
-  Entity::Load(_sdf);
+  // Create a DOM object to compute the resolved initial pose (with frame
+  // semantics)
+  // Only <model> is supported right now, <actor> is not supported.
+  if (_sdf->GetName() == "model")
+  {
+    const auto modelName = _sdf->Get<std::string>("name");
 
+    // If the model is initialized without a containing parent model or world,
+    // we create an isolated/standalone DOM object for the model. Otherwise,
+    // we'd obtain the model DOM from the parent DOM.
+    BasePtr parentEntity = this->GetParent();
+    if (nullptr != parentEntity && !modelName.empty())
+    {
+      auto parentModelPtr = boost::dynamic_pointer_cast<Model>(parentEntity);
+      if (parentModelPtr)
+      {
+        auto parentDom = parentModelPtr->GetSDFDom();
+        if (nullptr != parentDom)
+        {
+          this->modelSDFDom = parentDom->ModelByName(modelName);
+        }
+      }
+      else
+      {
+        // If the parent is not a model, we assume it's a world.
+        auto parentDom = this->GetWorld()->GetSDFDom();
+        if (nullptr != parentDom)
+        {
+          this->modelSDFDom = parentDom->ModelByName(modelName);
+        }
+      }
+    }
+
+    if (nullptr == this->modelSDFDom)
+    {
+      this->modelSDFDomIsolated = std::make_unique<sdf::Model>();
+      sdf::Errors errors = this->modelSDFDomIsolated->Load(_sdf);
+      auto sdfVersion =
+        ignition::math::SemanticVersion(_sdf->OriginalVersion());
+      // Only print out errors if the original SDFormat version does not support
+      // frame semantics
+      if (sdfVersion >= ignition::math::SemanticVersion(1, 7))
+      {
+        // Print errors and load the parts that worked.
+        for (const auto &error : errors)
+        {
+          if (common::isSdfFrameSemanticsError(error))
+          {
+            gzerr << error << "\n";
+          }
+        }
+      }
+      this->modelSDFDom = this->modelSDFDomIsolated.get();
+    }
+  }
+
+  Entity::Load(_sdf);
   this->jointPub = this->node->Advertise<msgs::Joint>("~/joint");
 
   this->SetStatic(this->sdf->Get<bool>("static"));
@@ -85,6 +143,11 @@ void Model::Load(sdf::ElementPtr _sdf)
     this->SetSelfCollide(this->sdf->Get<bool>("self_collide"));
   }
 
+  if (this->sdf->HasElement("enable_wind"))
+  {
+    this->SetWindMode(this->sdf->Get<bool>("enable_wind"));
+  }
+
   if (this->sdf->HasElement("allow_auto_disable"))
     this->SetAutoDisable(this->sdf->Get<bool>("allow_auto_disable"));
 
@@ -112,7 +175,7 @@ void Model::LoadLinks()
     while (linkElem)
     {
       // Create a new link
-      LinkPtr link = this->GetWorld()->GetPhysicsEngine()->CreateLink(
+      LinkPtr link = this->GetWorld()->Physics()->CreateLink(
           boost::static_pointer_cast<Model>(shared_from_this()));
 
       /// \TODO: canonical link is hardcoded to the first link.
@@ -178,7 +241,7 @@ void Model::LoadModels()
     while (modelElem)
     {
       // Create a new model
-      ModelPtr model = this->GetWorld()->GetPhysicsEngine()->CreateModel(
+      ModelPtr model = this->GetWorld()->Physics()->CreateModel(
           boost::static_pointer_cast<Model>(shared_from_this()));
       model->SetWorld(this->GetWorld());
       model->Load(modelElem);
@@ -235,9 +298,8 @@ void Model::LoadJoints()
 void Model::Init()
 {
   // Record the model's initial pose (for reseting)
-  math::Pose initPose = this->sdf->Get<math::Pose>("pose");
-  this->SetInitialRelativePose(initPose);
-  this->SetRelativePose(initPose);
+  this->SetInitialRelativePose(this->SDFPoseRelativeToParent());
+  this->SetRelativePose(this->SDFPoseRelativeToParent());
 
   // Initialize the bodies before the joints
   for (Base_V::iterator iter = this->children.begin();
@@ -287,18 +349,28 @@ void Model::Init()
 //////////////////////////////////////////////////
 void Model::Update()
 {
+  IGN_PROFILE("Model::Update");
   if (this->IsStatic())
     return;
 
+  IGN_PROFILE_BEGIN("lockMutex");
   boost::recursive_mutex::scoped_lock lock(this->updateMutex);
+  IGN_PROFILE_END();
 
+  IGN_PROFILE_BEGIN("jointUpdate");
   for (Joint_V::iterator jiter = this->joints.begin();
        jiter != this->joints.end(); ++jiter)
+  {
     (*jiter)->Update();
+  }
+  IGN_PROFILE_END();
 
+  IGN_PROFILE_BEGIN("jointControllerUpdate");
   if (this->jointController)
     this->jointController->Update();
+  IGN_PROFILE_END();
 
+  IGN_PROFILE_BEGIN("jointAnimations");
   if (!this->jointAnimations.empty())
   {
     common::NumericKeyFrame kf(0);
@@ -310,7 +382,7 @@ void Model::Update()
       iter->second->GetInterpolatedKeyFrame(kf);
 
       iter->second->AddTime(
-          (this->world->GetSimTime() - this->prevAnimationTime).Double());
+          (this->world->SimTime() - this->prevAnimationTime).Double());
 
       if (iter->second->GetTime() < iter->second->GetLength())
       {
@@ -332,11 +404,14 @@ void Model::Update()
       if (this->onJointAnimationComplete)
         this->onJointAnimationComplete();
     }
-    this->prevAnimationTime = this->world->GetSimTime();
+    this->prevAnimationTime = this->world->SimTime();
   }
+  IGN_PROFILE_END();
 
+  IGN_PROFILE_BEGIN("nestedModelUpdate");
   for (auto &model : this->models)
     model->Update();
+  IGN_PROFILE_END();
 }
 
 //////////////////////////////////////////////////
@@ -517,6 +592,11 @@ const sdf::ElementPtr Model::GetSDF()
   return Entity::GetSDF();
 }
 
+const sdf::Model *Model::GetSDFDom() const
+{
+  return this->modelSDFDom;
+}
+
 //////////////////////////////////////////////////
 const sdf::ElementPtr Model::UnscaledSDF()
 {
@@ -569,8 +649,7 @@ const sdf::ElementPtr Model::UnscaledSDF()
         }
         else if (geomElem->HasElement("mesh"))
         {
-          geomElem->GetElement("mesh")->GetElement("scale")->Set(
-              ignition::math::Vector3d::One);
+          // Keep mesh scale because meshes can't be scaled yet (issue #1473)
         }
 
         visualElem = visualElem->GetNextElement("visual");
@@ -613,8 +692,7 @@ const sdf::ElementPtr Model::UnscaledSDF()
         }
         else if (geomElem->HasElement("mesh"))
         {
-          geomElem->GetElement("mesh")->GetElement("scale")->Set(
-              ignition::math::Vector3d::One);
+          // Keep mesh scale because meshes can't be scaled yet (issue #1473)
         }
 
         collisionElem = collisionElem->GetNextElement("collision");
@@ -665,7 +743,7 @@ void Model::ResetPhysicsStates()
 }
 
 //////////////////////////////////////////////////
-void Model::SetLinearVel(const math::Vector3 &_vel)
+void Model::SetLinearVel(const ignition::math::Vector3d &_vel)
 {
   for (Link_V::iterator iter = this->links.begin();
        iter != this->links.end(); ++iter)
@@ -679,7 +757,7 @@ void Model::SetLinearVel(const math::Vector3 &_vel)
 }
 
 //////////////////////////////////////////////////
-void Model::SetAngularVel(const math::Vector3 &_vel)
+void Model::SetAngularVel(const ignition::math::Vector3d &_vel)
 {
   for (Link_V::iterator iter = this->links.begin();
        iter != this->links.end(); ++iter)
@@ -693,121 +771,94 @@ void Model::SetAngularVel(const math::Vector3 &_vel)
 }
 
 //////////////////////////////////////////////////
-void Model::SetLinearAccel(const math::Vector3 &_accel)
+ignition::math::Vector3d Model::RelativeLinearVel() const
 {
-  for (Link_V::iterator iter = this->links.begin();
-       iter != this->links.end(); ++iter)
+  if (this->GetLink("canonical"))
+    return this->GetLink("canonical")->RelativeLinearVel();
+  else
+    return ignition::math::Vector3d::Zero;
+}
+
+//////////////////////////////////////////////////
+ignition::math::Vector3d Model::WorldLinearVel() const
+{
+  if (this->GetLink("canonical"))
   {
-    if (*iter)
-    {
-      (*iter)->SetEnabled(true);
-      (*iter)->SetLinearAccel(_accel);
-    }
+    return this->GetLink("canonical")->WorldLinearVel();
   }
+  else
+    return ignition::math::Vector3d::Zero;
 }
 
 //////////////////////////////////////////////////
-void Model::SetAngularAccel(const math::Vector3 &_accel)
+ignition::math::Vector3d Model::RelativeAngularVel() const
 {
-  for (Link_V::iterator iter = this->links.begin();
-       iter != this->links.end(); ++iter)
+  if (this->GetLink("canonical"))
+    return this->GetLink("canonical")->RelativeAngularVel();
+  else
+    return ignition::math::Vector3d::Zero;
+}
+
+//////////////////////////////////////////////////
+ignition::math::Vector3d Model::WorldAngularVel() const
+{
+  if (this->GetLink("canonical"))
+    return this->GetLink("canonical")->WorldAngularVel();
+  else
+    return ignition::math::Vector3d::Zero;
+}
+
+
+//////////////////////////////////////////////////
+ignition::math::Vector3d Model::RelativeLinearAccel() const
+{
+  if (this->GetLink("canonical"))
+    return this->GetLink("canonical")->RelativeLinearAccel();
+  else
+    return ignition::math::Vector3d::Zero;
+}
+
+//////////////////////////////////////////////////
+ignition::math::Vector3d Model::WorldLinearAccel() const
+{
+  if (this->GetLink("canonical"))
+    return this->GetLink("canonical")->WorldLinearAccel();
+  else
+    return ignition::math::Vector3d::Zero;
+}
+
+//////////////////////////////////////////////////
+ignition::math::Vector3d Model::RelativeAngularAccel() const
+{
+  if (this->GetLink("canonical"))
+    return this->GetLink("canonical")->RelativeAngularAccel();
+  else
+    return ignition::math::Vector3d::Zero;
+}
+
+//////////////////////////////////////////////////
+ignition::math::Vector3d Model::WorldAngularAccel() const
+{
+  if (this->GetLink("canonical"))
+    return this->GetLink("canonical")->WorldAngularAccel();
+  else
+    return ignition::math::Vector3d::Zero;
+}
+
+//////////////////////////////////////////////////
+ignition::math::AxisAlignedBox Model::BoundingBox() const
+{
+  ignition::math::AxisAlignedBox box;
+
+  box.Min().Set(FLT_MAX, FLT_MAX, FLT_MAX);
+  box.Max().Set(-FLT_MAX, -FLT_MAX, -FLT_MAX);
+
+  for (const auto &iter : this->links)
   {
-    if (*iter)
+    if (iter)
     {
-      (*iter)->SetEnabled(true);
-      (*iter)->SetAngularAccel(_accel);
-    }
-  }
-}
-
-//////////////////////////////////////////////////
-math::Vector3 Model::GetRelativeLinearVel() const
-{
-  if (this->GetLink("canonical"))
-    return this->GetLink("canonical")->GetRelativeLinearVel();
-  else
-    return math::Vector3(0, 0, 0);
-}
-
-//////////////////////////////////////////////////
-math::Vector3 Model::GetWorldLinearVel() const
-{
-  if (this->GetLink("canonical"))
-    return this->GetLink("canonical")->GetWorldLinearVel();
-  else
-    return math::Vector3(0, 0, 0);
-}
-
-//////////////////////////////////////////////////
-math::Vector3 Model::GetRelativeAngularVel() const
-{
-  if (this->GetLink("canonical"))
-    return this->GetLink("canonical")->GetRelativeAngularVel();
-  else
-    return math::Vector3(0, 0, 0);
-}
-
-//////////////////////////////////////////////////
-math::Vector3 Model::GetWorldAngularVel() const
-{
-  if (this->GetLink("canonical"))
-    return this->GetLink("canonical")->GetWorldAngularVel();
-  else
-    return math::Vector3(0, 0, 0);
-}
-
-
-//////////////////////////////////////////////////
-math::Vector3 Model::GetRelativeLinearAccel() const
-{
-  if (this->GetLink("canonical"))
-    return this->GetLink("canonical")->GetRelativeLinearAccel();
-  else
-    return math::Vector3(0, 0, 0);
-}
-
-//////////////////////////////////////////////////
-math::Vector3 Model::GetWorldLinearAccel() const
-{
-  if (this->GetLink("canonical"))
-    return this->GetLink("canonical")->GetWorldLinearAccel();
-  else
-    return math::Vector3(0, 0, 0);
-}
-
-//////////////////////////////////////////////////
-math::Vector3 Model::GetRelativeAngularAccel() const
-{
-  if (this->GetLink("canonical"))
-    return this->GetLink("canonical")->GetRelativeAngularAccel();
-  else
-    return math::Vector3(0, 0, 0);
-}
-
-//////////////////////////////////////////////////
-math::Vector3 Model::GetWorldAngularAccel() const
-{
-  if (this->GetLink("canonical"))
-    return this->GetLink("canonical")->GetWorldAngularAccel();
-  else
-    return math::Vector3(0, 0, 0);
-}
-
-//////////////////////////////////////////////////
-math::Box Model::GetBoundingBox() const
-{
-  math::Box box;
-
-  box.min.Set(FLT_MAX, FLT_MAX, FLT_MAX);
-  box.max.Set(-FLT_MAX, -FLT_MAX, -FLT_MAX);
-
-  for (Link_V::const_iterator iter = this->links.begin();
-       iter != this->links.end(); ++iter)
-  {
-    if (*iter)
-    {
-      math::Box linkBox;
-      linkBox = (*iter)->GetBoundingBox();
+      ignition::math::AxisAlignedBox linkBox;
+      linkBox = iter->BoundingBox();
       box += linkBox;
     }
   }
@@ -912,7 +963,7 @@ void Model::LoadJoint(sdf::ElementPtr _sdf)
 
   std::string stype = _sdf->Get<std::string>("type");
 
-  joint = this->GetWorld()->GetPhysicsEngine()->CreateJoint(stype,
+  joint = this->GetWorld()->Physics()->CreateJoint(stype,
      boost::static_pointer_cast<Model>(shared_from_this()));
   if (!joint)
   {
@@ -950,18 +1001,51 @@ void Model::LoadGripper(sdf::ElementPtr _sdf)
   this->grippers.push_back(gripper);
 }
 
+//////////////////////////////////////////////////
+std::vector<std::string> Model::SensorScopedName(
+  const std::string &_name) const
+{
+  std::vector<std::string> names;
+  for (Link_V::const_iterator iter = this->links.begin();
+       iter != this->links.end(); ++iter)
+  {
+    for (unsigned int j = 0; j < (*iter)->GetSensorCount(); ++j)
+    {
+      const auto sensorName = (*iter)->GetSensorName(j);
+      if (sensorName.size() < _name.size())
+      {
+        continue;
+      }
+      if (sensorName.substr(sensorName.size() - _name.size(), _name.size()) ==
+          _name)
+      {
+        names.push_back(sensorName);
+      }
+    }
+  }
+
+  return names;
+}
+
 //////////////////////////////////////////////////
 void Model::LoadPlugins()
+{
+  this->LoadPlugins(30);
+}
+
+//////////////////////////////////////////////////
+void Model::LoadPlugins(unsigned int _timeout)
 {
   // Check to see if we need to load any model plugins
   if (this->GetPluginCount() > 0)
   {
     int iterations = 0;
+    const int maxIterations = _timeout * 10;
 
     // Wait for the sensors to be initialized before loading
     // plugins, if there are any sensors
     while (this->GetSensorCount() > 0 && !this->world->SensorsInitialized() &&
-           iterations < 50)
+           iterations < maxIterations)
     {
       common::Time::MSleep(100);
       iterations++;
@@ -969,7 +1053,7 @@ void Model::LoadPlugins()
 
     // Load the plugins if the sensors have been loaded, or if there
     // are no sensors attached to the model.
-    if (iterations < 50)
+    if (iterations < maxIterations)
     {
       // Load the plugins
       sdf::ElementPtr pluginElem = this->sdf->GetElement("plugin");
@@ -983,12 +1067,12 @@ void Model::LoadPlugins()
     {
       gzerr << "Sensors failed to initialize when loading model["
         << this->GetName() << "] via the factory mechanism."
-        << "Plugins for the model will not be loaded.\n";
+        << " Plugins for the model will not be loaded.\n";
     }
   }
 
   for (auto &model : this->models)
-    model->LoadPlugins();
+    model->LoadPlugins(_timeout);
 }
 
 //////////////////////////////////////////////////
@@ -1146,11 +1230,12 @@ void Model::SetLaserRetro(const float _retro)
 //////////////////////////////////////////////////
 void Model::FillMsg(msgs::Model &_msg)
 {
-  ignition::math::Pose3d relPose = this->GetRelativePose().Ign();
+  ignition::math::Pose3d relPose = this->RelativePose();
 
   _msg.set_name(this->GetScopedName());
   _msg.set_is_static(this->IsStatic());
   _msg.set_self_collide(this->GetSelfCollide());
+  _msg.set_enable_wind(this->WindMode());
   msgs::Set(_msg.mutable_pose(), relPose);
   _msg.set_id(this->GetId());
   msgs::Set(_msg.mutable_scale(), this->scale);
@@ -1171,6 +1256,18 @@ void Model::FillMsg(msgs::Model &_msg)
   {
     model->FillMsg(*_msg.add_model());
   }
+
+  if (this->sdf->HasElement("plugin"))
+  {
+    sdf::ElementPtr pluginElem = this->sdf->GetElement("plugin");
+    while (pluginElem)
+    {
+      auto pluginMsg = _msg.add_plugin();
+      pluginMsg->CopyFrom(msgs::PluginFromSDF(pluginElem));
+
+      pluginElem = pluginElem->GetNextElement("plugin");
+    }
+  }
 }
 
 //////////////////////////////////////////////////
@@ -1189,7 +1286,8 @@ void Model::ProcessMsg(const msgs::Model &_msg)
     return;
   }
 
-  this->SetName(this->world->StripWorldName(_msg.name()));
+  this->SetName(this->StripParentScopedName(_msg.name()));
+
   if (_msg.has_pose())
     this->SetWorldPose(msgs::ConvertIgn(_msg.pose()));
   for (int i = 0; i < _msg.link_size(); i++)
@@ -1204,6 +1302,9 @@ void Model::ProcessMsg(const msgs::Model &_msg)
 
   if (_msg.has_scale())
     this->SetScale(msgs::ConvertIgn(_msg.scale()));
+
+  if (_msg.has_enable_wind())
+    this->SetWindMode(_msg.enable_wind());
 }
 
 //////////////////////////////////////////////////
@@ -1218,7 +1319,7 @@ void Model::SetJointAnimation(
     this->jointAnimations[iter->first] = iter->second;
   }
   this->onJointAnimationComplete = _onComplete;
-  this->prevAnimationTime = this->world->GetSimTime();
+  this->prevAnimationTime = this->world->SimTime();
 }
 
 //////////////////////////////////////////////////
@@ -1231,7 +1332,7 @@ void Model::StopAnimation()
 }
 
 //////////////////////////////////////////////////
-void Model::AttachStaticModel(ModelPtr &_model, math::Pose _offset)
+void Model::AttachStaticModel(ModelPtr &_model, ignition::math::Pose3d _offset)
 {
   if (!_model->IsStatic())
   {
@@ -1260,10 +1361,10 @@ void Model::DetachStaticModel(const std::string &_modelName)
 //////////////////////////////////////////////////
 void Model::OnPoseChange()
 {
-  math::Pose p;
+  ignition::math::Pose3d p;
   for (unsigned int i = 0; i < this->attachedModels.size(); i++)
   {
-    p = this->GetWorldPose();
+    p = this->WorldPose();
     p += this->attachedModelsOffset[i];
     this->attachedModels[i]->SetWorldPose(p, true);
   }
@@ -1272,7 +1373,7 @@ void Model::OnPoseChange()
 //////////////////////////////////////////////////
 void Model::SetState(const ModelState &_state)
 {
-  this->SetWorldPose(_state.GetPose(), true);
+  this->SetWorldPose(_state.Pose(), true);
   this->SetScale(_state.Scale(), true);
 
   LinkState_M linkStates = _state.GetLinkStates();
@@ -1305,27 +1406,21 @@ void Model::SetState(const ModelState &_state)
   // }
 }
 
-/////////////////////////////////////////////////
-void Model::SetScale(const math::Vector3 &_scale)
-{
-  this->SetScale(_scale.Ign());
-}
-
 /////////////////////////////////////////////////
 void Model::SetScale(const ignition::math::Vector3d &_scale,
       const bool _publish)
 {
-  if (this->scale == _scale)
-    return;
-
-  this->scale = _scale;
-
-  Base_V::iterator iter;
-  for (iter = this->children.begin(); iter != this->children.end(); ++iter)
+  if (this->scale != _scale)
   {
-    if (*iter && (*iter)->HasType(LINK))
+    this->scale = _scale;
+
+    Base_V::iterator iter;
+    for (iter = this->children.begin(); iter != this->children.end(); ++iter)
     {
-      boost::static_pointer_cast<Link>(*iter)->SetScale(_scale);
+      if (*iter && (*iter)->HasType(LINK))
+      {
+        boost::static_pointer_cast<Link>(*iter)->SetScale(_scale);
+      }
     }
   }
 
@@ -1360,7 +1455,8 @@ void Model::SetEnabled(bool _enabled)
 }
 
 /////////////////////////////////////////////////
-void Model::SetLinkWorldPose(const math::Pose &_pose, std::string _linkName)
+void Model::SetLinkWorldPose(const ignition::math::Pose3d &_pose,
+    std::string _linkName)
 {
   // look for link matching link name
   LinkPtr link = this->GetLink(_linkName);
@@ -1372,12 +1468,13 @@ void Model::SetLinkWorldPose(const math::Pose &_pose, std::string _linkName)
 }
 
 /////////////////////////////////////////////////
-void Model::SetLinkWorldPose(const math::Pose &_pose, const LinkPtr &_link)
+void Model::SetLinkWorldPose(const ignition::math::Pose3d &_pose,
+    const LinkPtr &_link)
 {
-  math::Pose linkPose = _link->GetWorldPose();
-  math::Pose currentModelPose = this->GetWorldPose();
-  math::Pose linkRelPose = currentModelPose - linkPose;
-  math::Pose targetModelPose =  linkRelPose * _pose;
+  ignition::math::Pose3d linkPose = _link->WorldPose();
+  ignition::math::Pose3d currentModelPose = this->WorldPose();
+  ignition::math::Pose3d linkRelPose = currentModelPose - linkPose;
+  ignition::math::Pose3d targetModelPose =  linkRelPose * _pose;
   this->SetWorldPose(targetModelPose);
 }
 
@@ -1461,7 +1558,7 @@ double Model::GetWorldEnergyPotential() const
   for (Joint_V::const_iterator iter = this->joints.begin();
     iter != this->joints.end(); ++iter)
   {
-    for (unsigned int j = 0; j < (*iter)->GetAngleCount(); ++j)
+    for (unsigned int j = 0; j < (*iter)->DOF(); ++j)
     {
       e += (*iter)->GetWorldEnergyPotentialSpring(j);
     }
@@ -1500,12 +1597,11 @@ gazebo::physics::JointPtr Model::CreateJoint(
            << "], skipping creating joint.\n";
     return joint;
   }
-  joint =
-    this->world->GetPhysicsEngine()->CreateJoint(_type, shared_from_this());
+  joint = this->world->Physics()->CreateJoint(_type, shared_from_this());
   joint->SetName(_name);
   joint->Attach(_parent, _child);
   // need to call Joint::Load to clone Joint::sdfJoint into Joint::sdf
-  joint->Load(_parent, _child, gazebo::math::Pose());
+  joint->Load(_parent, _child, ignition::math::Pose3d::Zero);
   this->joints.push_back(joint);
   return joint;
 }
@@ -1572,6 +1668,83 @@ bool Model::RemoveJoint(const std::string &_name)
   }
 }
 
+/////////////////////////////////////////////////
+void Model::SetWindMode(const bool _enable)
+{
+  this->sdf->GetElement("enable_wind")->Set(_enable);
+  for (auto &link : this->links)
+    link->SetWindMode(_enable);
+}
+
+/////////////////////////////////////////////////
+bool Model::WindMode() const
+{
+  return this->sdf->Get<bool>("enable_wind");
+}
+
+/////////////////////////////////////////////////
+void Model::RegisterIntrospectionItems()
+{
+  auto uri = this->URI();
+
+  // Callbacks.
+  auto fModelPose = [this]()
+  {
+    return this->WorldPose();
+  };
+
+  auto fModelLinVel = [this]()
+  {
+    return this->WorldLinearVel();
+  };
+
+  auto fModelAngVel = [this]()
+  {
+    return this->WorldAngularVel();
+  };
+
+  auto fModelLinAcc = [this]()
+  {
+    return this->WorldLinearAccel();
+  };
+
+  auto fModelAngAcc = [this]()
+  {
+    return this->WorldAngularAccel();
+  };
+
+  // Register items.
+  common::URI poseURI(uri);
+  poseURI.Query().Insert("p", "pose3d/world_pose");
+  this->introspectionItems.push_back(poseURI);
+  gazebo::util::IntrospectionManager::Instance()->Register
+      <ignition::math::Pose3d>(poseURI.Str(), fModelPose);
+
+  common::URI linVelURI(uri);
+  linVelURI.Query().Insert("p", "vector3d/world_linear_velocity");
+  this->introspectionItems.push_back(linVelURI);
+  gazebo::util::IntrospectionManager::Instance()->Register
+      <ignition::math::Vector3d>(linVelURI.Str(), fModelLinVel);
+
+  common::URI angVelURI(uri);
+  angVelURI.Query().Insert("p", "vector3d/world_angular_velocity");
+  this->introspectionItems.push_back(angVelURI);
+  gazebo::util::IntrospectionManager::Instance()->Register
+      <ignition::math::Vector3d>(angVelURI.Str(), fModelAngVel);
+
+  common::URI linAccURI(uri);
+  linAccURI.Query().Insert("p", "vector3d/world_linear_acceleration");
+  this->introspectionItems.push_back(linAccURI);
+  gazebo::util::IntrospectionManager::Instance()->Register
+      <ignition::math::Vector3d>(linAccURI.Str(), fModelLinAcc);
+
+  common::URI angAccURI(uri);
+  angAccURI.Query().Insert("p", "vector3d/world_angular_acceleration");
+  this->introspectionItems.push_back(angAccURI);
+  gazebo::util::IntrospectionManager::Instance()->Register
+      <ignition::math::Vector3d>(angAccURI.Str(), fModelAngAcc);
+}
+
 /////////////////////////////////////////////////
 LinkPtr Model::CreateLink(const std::string &_name)
 {
@@ -1584,10 +1757,112 @@ LinkPtr Model::CreateLink(const std::string &_name)
     return link;
   }
 
-  link = this->world->GetPhysicsEngine()->CreateLink(shared_from_this());
+  link = this->world->Physics()->CreateLink(shared_from_this());
 
   link->SetName(_name);
   this->links.push_back(link);
 
   return link;
 }
+
+//////////////////////////////////////////////////
+void Model::PluginInfo(const common::URI &_pluginUri,
+    ignition::msgs::Plugin_V &_plugins, bool &_success)
+{
+  _plugins.clear_plugins();
+  _success = false;
+
+  if (!_pluginUri.Valid())
+  {
+    gzwarn << "URI [" << _pluginUri.Str() << "] is not valid." << std::endl;
+    return;
+  }
+
+  if (!_pluginUri.Path().Contains(this->URI().Path()))
+  {
+    gzwarn << "Plugin [" << _pluginUri.Str() << "] does not match world [" <<
+        this->URI().Str() << "]" << std::endl;
+    return;
+  }
+
+  auto parts = common::split(_pluginUri.Path().Str(), "/");
+  auto myParts = common::split(this->URI().Path().Str(), "/");
+
+  // Iterate over parts following this model
+  for (size_t i = myParts.size(); i < parts.size(); i = i+2)
+  {
+    if (parts[i] == "model")
+    {
+      // Look in nested models
+      auto model = this->NestedModel(parts[i+1]);
+
+      if (!model)
+      {
+        gzwarn << "Model [" << parts[i+1] << "] not found in model [" <<
+            this->GetName() << "]" << std::endl;
+        return;
+      }
+
+      model->PluginInfo(_pluginUri, _plugins, _success);
+      return;
+    }
+    // Look for plugin
+    else if (parts[i] == "plugin")
+    {
+      // Return empty vector
+      if (!this->sdf->HasElement("plugin"))
+      {
+        _success = true;
+        return;
+      }
+
+      // Find correct plugin
+      auto pluginElem = this->sdf->GetElement("plugin");
+      while (pluginElem)
+      {
+        auto pluginName = pluginElem->Get<std::string>("name");
+
+        // If asking for a specific plugin, skip all other plugins
+        if (i+1 < parts.size() && parts[i+1] != pluginName)
+        {
+          pluginElem = pluginElem->GetNextElement("plugin");
+          continue;
+        }
+
+        // Get plugin info from SDF
+        auto pluginMsg = _plugins.add_plugins();
+        pluginMsg->CopyFrom(util::Convert<ignition::msgs::Plugin>(pluginElem));
+
+        pluginElem = pluginElem->GetNextElement("plugin");
+      }
+
+      // If asking for a specific plugin and it wasn't found
+      if (i+1 < parts.size() && _plugins.plugins_size() == 0)
+      {
+        gzwarn << "Plugin [" << parts[i+1] << "] not found in model [" <<
+            this->URI().Str() << "]" << std::endl;
+        return;
+      }
+      _success = true;
+      return;
+    }
+    else
+    {
+      gzwarn << "Segment [" << parts[i] << "] in [" << _pluginUri.Str() <<
+         "] cannot be handled." << std::endl;
+      return;
+    }
+  }
+
+  gzwarn << "Couldn't get information for plugin [" << _pluginUri.Str() << "]"
+      << std::endl;
+}
+
+std::optional<sdf::SemanticPose> Model::SDFSemanticPose() const
+{
+  if (nullptr != this->modelSDFDom)
+  {
+    return this->modelSDFDom->SemanticPose();
+  }
+  return std::nullopt;
+}
