@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2014 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,6 +18,8 @@
 #include <limits>
 #include <string>
 
+#include <ignition/math/Helpers.hh>
+
 #include "gazebo/common/Assert.hh"
 #include "gazebo/common/Console.hh"
 #include "gazebo/common/Exception.hh"
@@ -97,9 +99,9 @@ using namespace physics;
 
 //////////////////////////////////////////////////
 BulletScrewJoint::BulletScrewJoint(btDynamicsWorld *_world, BasePtr _parent)
-    : ScrewJoint<BulletJoint>(_parent), bulletScrew(NULL)
+    : ScrewJoint<BulletJoint>(_parent), bulletScrew(nullptr)
 {
-  GZ_ASSERT(_world, "bullet world pointer is NULL");
+  GZ_ASSERT(_world, "bullet world pointer is null");
   this->bulletWorld = _world;
 }
 
@@ -116,15 +118,16 @@ void BulletScrewJoint::Load(sdf::ElementPtr _sdf)
 }
 
 //////////////////////////////////////////////////
-math::Vector3 BulletScrewJoint::GetAnchor(unsigned int /*index*/) const
+ignition::math::Vector3d BulletScrewJoint::Anchor(
+    const unsigned int /*index*/) const
 {
-  gzerr << "BulletScrewJoint::GetAnchor not implemented, return 0 vector.\n";
-  return math::Vector3();
+  gzerr << "BulletScrewJoint::Anchor not implemented, return 0 vector.\n";
+  return ignition::math::Vector3d::Zero;
 }
 
 //////////////////////////////////////////////////
-void BulletScrewJoint::SetAnchor(unsigned int /*index*/,
-    const math::Vector3 &/*_anchor*/)
+void BulletScrewJoint::SetAnchor(const unsigned int /*index*/,
+    const ignition::math::Vector3d &/*_anchor*/)
 {
   gzerr << "BulletScrewJoint::SetAnchor not implemented.\n";
 }
@@ -140,8 +143,8 @@ void BulletScrewJoint::Init()
     boost::static_pointer_cast<BulletLink>(this->parentLink);
 
   // Get axis unit vector (expressed in world frame).
-  math::Vector3 axis = this->initialWorldAxis;
-  if (axis == math::Vector3::Zero)
+  ignition::math::Vector3d axis = this->initialWorldAxis;
+  if (axis == ignition::math::Vector3d::Zero)
   {
     gzerr << "axis must have non-zero length, resetting to 0 0 1\n";
     axis.Set(0, 0, 1);
@@ -149,8 +152,8 @@ void BulletScrewJoint::Init()
 
   // Local variables used to compute pivots and axes in body-fixed frames
   // for the parent and child links.
-  math::Vector3 pivotParent, pivotChild, axisParent, axisChild;
-  math::Pose pose;
+  ignition::math::Vector3d pivotParent, pivotChild, axisParent, axisChild;
+  ignition::math::Pose3d pose;
   btTransform frameParent, frameChild;
   btVector3 axis2, axis3;
 
@@ -162,45 +165,56 @@ void BulletScrewJoint::Init()
   // Check if parentLink exists. If not, the parent will be the world.
   if (this->parentLink)
   {
-    // Compute relative pose between joint anchor and CoG of parent link.
-    pose = this->parentLink->GetWorldCoGPose();
+    // Compute relative pose between joint anchor and inertial frame of parent.
+    pose = this->parentLink->WorldInertialPose();
     // Subtract CoG position from anchor position, both in world frame.
-    pivotParent -= pose.pos;
-    // Rotate pivot offset and axis into body-fixed frame of parent.
-    pivotParent = pose.rot.RotateVectorReverse(pivotParent);
+    pivotParent -= pose.Pos();
+    // Rotate pivot offset and axis into body-fixed inertial frame of parent.
+    pivotParent = pose.Rot().RotateVectorReverse(pivotParent);
     frameParent.setOrigin(BulletTypes::ConvertVector3(pivotParent));
-    axisParent = pose.rot.RotateVectorReverse(axis);
+    axisParent = pose.Rot().RotateVectorReverse(axis);
     axisParent = axisParent.Normalize();
     // The following math is based on btHingeConstraint.cpp:95-115
     btPlaneSpace1(BulletTypes::ConvertVector3(axisParent), axis2, axis3);
     frameParent.getBasis().setValue(
-      axisParent.x, axis2.x(), axis3.x(),
-      axisParent.y, axis2.y(), axis3.y(),
-      axisParent.z, axis2.z(), axis3.z());
+      axisParent.X(), axis2.x(), axis3.x(),
+      axisParent.Y(), axis2.y(), axis3.y(),
+      axisParent.Z(), axis2.z(), axis3.z());
   }
   // Check if childLink exists. If not, the child will be the world.
   if (this->childLink)
   {
-    // Compute relative pose between joint anchor and CoG of child link.
-    pose = this->childLink->GetWorldCoGPose();
+    // Compute relative pose between joint anchor and inertial frame of child.
+    pose = this->childLink->WorldInertialPose();
     // Subtract CoG position from anchor position, both in world frame.
-    pivotChild -= pose.pos;
-    // Rotate pivot offset and axis into body-fixed frame of child.
-    pivotChild = pose.rot.RotateVectorReverse(pivotChild);
+    pivotChild -= pose.Pos();
+    // Rotate pivot offset and axis into body-fixed inertial frame of child.
+    pivotChild = pose.Rot().RotateVectorReverse(pivotChild);
     frameChild.setOrigin(BulletTypes::ConvertVector3(pivotChild));
-    axisChild = pose.rot.RotateVectorReverse(axis);
+    axisChild = pose.Rot().RotateVectorReverse(axis);
     axisChild = axisChild.Normalize();
     // The following math is based on btHingeConstraint.cpp:95-115
     btPlaneSpace1(BulletTypes::ConvertVector3(axisChild), axis2, axis3);
     frameChild.getBasis().setValue(
-      axisChild.x, axis2.x(), axis3.x(),
-      axisChild.y, axis2.y(), axis3.y(),
-      axisChild.z, axis2.z(), axis3.z());
+      axisChild.X(), axis2.x(), axis3.x(),
+      axisChild.Y(), axis2.y(), axis3.y(),
+      axisChild.Z(), axis2.z(), axis3.z());
   }
 
   // If both links exist, then create a joint between the two links.
   if (bulletChildLink && bulletParentLink)
   {
+    // Parent and child constraint frames generated with btPlaneSpace1 may
+    // differ by 90 degrees because they are underdetermined (only one axis
+    // given). Here we set the child constraint frame by taking the parent
+    // constraint frame (in the frame of the parent Bullet link) and rotating
+    // it into the frame of the child Bullet link. This ensures that the
+    // constraint frames are initially aligned.
+    pose = ignition::math::Pose3d(pivotChild,
+        this->childLink->WorldInertialPose().Rot().Inverse() *
+        this->parentLink->WorldInertialPose().Rot() *
+        BulletTypes::ConvertPoseIgn(frameParent).Rot());
+    frameChild = BulletTypes::ConvertPose(pose);
     this->bulletScrew = new btScrewConstraint(
         *bulletParentLink->GetBulletLink(),
         *bulletChildLink->GetBulletLink(),
@@ -235,7 +249,7 @@ void BulletScrewJoint::Init()
 
   // Apply joint translation limits here.
   // TODO: velocity and effort limits.
-  GZ_ASSERT(this->sdf != NULL, "Joint sdf member is NULL");
+  GZ_ASSERT(this->sdf != nullptr, "Joint sdf member is null");
   sdf::ElementPtr limitElem;
   limitElem = this->sdf->GetElement("axis")->GetElement("limit");
   // joint limit is set on the revolute dof in sdf,
@@ -245,7 +259,7 @@ void BulletScrewJoint::Init()
   this->bulletScrew->setUpperAngLimit(upper);
   // enforce linear dof in bullet.
   double tp = this->threadPitch;
-  if (math::equal(tp, 0.0))
+  if (ignition::math::equal(tp, 0.0))
   {
     gzerr << "thread pitch should not be zero (joint is a slider?)"
           << " using thread pitch = 1.0e6\n";
@@ -266,7 +280,7 @@ void BulletScrewJoint::Init()
   this->constraint = this->bulletScrew;
 
   // Add the joint to the world
-  GZ_ASSERT(this->bulletWorld, "bullet world pointer is NULL");
+  GZ_ASSERT(this->bulletWorld, "bullet world pointer is null");
   this->bulletWorld->addConstraint(this->constraint);
 
   // Allows access to impulse
@@ -280,36 +294,35 @@ void BulletScrewJoint::Init()
 double BulletScrewJoint::GetVelocity(unsigned int /*_index*/) const
 {
   double result = 0;
-  math::Vector3 globalAxis = this->GetGlobalAxis(0);
+  ignition::math::Vector3d globalAxis = this->GlobalAxis(0);
   if (this->childLink)
-    result += globalAxis.Dot(this->childLink->GetWorldLinearVel());
+    result += globalAxis.Dot(this->childLink->WorldLinearVel());
   if (this->parentLink)
-    result -= globalAxis.Dot(this->parentLink->GetWorldLinearVel());
+    result -= globalAxis.Dot(this->parentLink->WorldLinearVel());
   return result;
 }
 
 //////////////////////////////////////////////////
 void BulletScrewJoint::SetVelocity(unsigned int _index, double _vel)
 {
-  math::Vector3 desiredVel;
+  ignition::math::Vector3d desiredVel;
   if (this->parentLink)
-    desiredVel = this->parentLink->GetWorldLinearVel();
-  desiredVel += _vel * this->GetGlobalAxis(_index);
+    desiredVel = this->parentLink->WorldLinearVel();
+  desiredVel += _vel * this->GlobalAxis(_index);
   if (this->childLink)
     this->childLink->SetLinearVel(desiredVel);
 }
 
 //////////////////////////////////////////////////
-void BulletScrewJoint::SetAxis(unsigned int /*_index*/,
-    const math::Vector3 &_axis)
+void BulletScrewJoint::SetAxis(const unsigned int /*_index*/,
+    const ignition::math::Vector3d &_axis)
 {
   // Note that _axis is given in a world frame,
   // but bullet uses a body-fixed frame
   if (!this->bulletScrew)
   {
     // this hasn't been initialized yet, store axis in initialWorldAxis
-    math::Quaternion axisFrame = this->GetAxisFrame(0);
-    this->initialWorldAxis = axisFrame.RotateVector(_axis);
+    this->initialWorldAxis = this->WorldPose().Rot() * _axis;
   }
   else
   {
@@ -392,47 +405,27 @@ void BulletScrewJoint::SetForceImpl(unsigned int _index, double _force)
 }
 
 //////////////////////////////////////////////////
-void BulletScrewJoint::SetMaxForce(unsigned int /*_index*/, double _force)
+void BulletScrewJoint::SetUpperLimit(const unsigned int _index,
+                                     const double _limit)
 {
-  if (this->bulletScrew)
-    this->bulletScrew->setMaxLinMotorForce(_force);
-  else
-    gzerr << "bulletScrew not created yet\n";
-}
-
-//////////////////////////////////////////////////
-double BulletScrewJoint::GetMaxForce(unsigned int /*index*/)
-{
-  double result = 0;
-  if (this->bulletScrew)
-    result = this->bulletScrew->getMaxLinMotorForce();
-  else
-    gzerr << "bulletScrew not created yet\n";
-  return result;
-}
-
-//////////////////////////////////////////////////
-bool BulletScrewJoint::SetHighStop(unsigned int _index,
-                      const math::Angle &_angle)
-{
-  Joint::SetHighStop(0, _angle);
+  Joint::SetUpperLimit(0, _limit);
 
   // bulletScrew axial rotation is backward
   if (!this->bulletScrew)
   {
     gzerr << "bulletScrew not created yet.\n";
-    return false;
+    return;
   }
 
   if (_index == 0)
   {
     // _index = 0: angular constraint
     double upperAng = this->bulletScrew->getUpperAngLimit();
-    this->bulletScrew->setUpperAngLimit(std::max(upperAng, _angle.Radian()));
+    this->bulletScrew->setUpperAngLimit(std::max(upperAng, _limit));
 
     // set corresponding linear constraints
     double tp = this->threadPitch;
-    if (math::equal(tp, 0.0))
+    if (ignition::math::equal(tp, 0.0))
     {
       gzwarn << "thread pitch should not be zero (joint is a slider?)"
              << " using thread pitch = 1.0e6\n";
@@ -443,22 +436,21 @@ bool BulletScrewJoint::SetHighStop(unsigned int _index,
     {
       double lowerLin = this->bulletScrew->getLowerLinLimit();
       this->bulletScrew->setUpperLinLimit(std::max(lowerLin,
-        _angle.Radian()/tp));
+        _limit/tp));
     }
     else
     {
       // flip upper lower because thread pitch is negative
       double upperLin = this->bulletScrew->getUpperLinLimit();
       this->bulletScrew->setLowerLinLimit(std::min(upperLin,
-        _angle.Radian()/tp));
+        _limit/tp));
     }
-    return true;
   }
   else if (_index == 1)
   {
     // _index = 1: linear constraint
     double lowerLin = this->bulletScrew->getLowerLinLimit();
-    this->bulletScrew->setUpperLinLimit(std::max(lowerLin, _angle.Radian()));
+    this->bulletScrew->setUpperLinLimit(std::max(lowerLin, _limit));
 
     // set corresponding angular constraints
     double tp = this->threadPitch;
@@ -467,35 +459,33 @@ bool BulletScrewJoint::SetHighStop(unsigned int _index,
     {
       double lowerAng = this->bulletScrew->getLowerAngLimit();
       this->bulletScrew->setUpperAngLimit(std::max(lowerAng,
-        _angle.Radian()*tp));
+        _limit*tp));
     }
     else
     {
       // flip upper lower because thread pitch is negative
       double upperAng = this->bulletScrew->getUpperAngLimit();
       this->bulletScrew->setLowerAngLimit(std::min(upperAng,
-        _angle.Radian()*tp));
+        _limit*tp));
     }
-    return true;
   }
   else
   {
     gzerr << "Invalid index [" << _index << "]\n";
-    return false;
   }
 }
 
 //////////////////////////////////////////////////
-bool BulletScrewJoint::SetLowStop(unsigned int _index,
-                     const math::Angle &_angle)
+void BulletScrewJoint::SetLowerLimit(const unsigned int _index,
+                                     const double _limit)
 {
-  Joint::SetLowStop(0, _angle);
+  Joint::SetLowerLimit(0, _limit);
 
   // bulletScrew axial rotation is backward
   if (!this->bulletScrew)
   {
     gzerr << "bulletScrew not created yet.\n";
-    return false;
+    return;
   }
 
   // bulletScrew axial rotation is backward
@@ -503,11 +493,11 @@ bool BulletScrewJoint::SetLowStop(unsigned int _index,
   {
     // _index = 0: angular constraint
     double upperAng = this->bulletScrew->getUpperAngLimit();
-    this->bulletScrew->setLowerAngLimit(std::min(upperAng, _angle.Radian()));
+    this->bulletScrew->setLowerAngLimit(std::min(upperAng, _limit));
 
     // set corresponding linear constraints
     double tp = this->threadPitch;
-    if (math::equal(tp, 0.0))
+    if (ignition::math::equal(tp, 0.0))
     {
       gzerr << "thread pitch should not be zero (joint is a slider?)"
             << " using thread pitch = 1.0e6\n";
@@ -518,22 +508,21 @@ bool BulletScrewJoint::SetLowStop(unsigned int _index,
     {
       double upperLin = this->bulletScrew->getUpperLinLimit();
       this->bulletScrew->setLowerLinLimit(std::min(upperLin,
-        _angle.Radian()/tp));
+        _limit/tp));
     }
     else
     {
         // flip upper lower because thread pitch is negative
       double lowerLin = this->bulletScrew->getLowerLinLimit();
       this->bulletScrew->setUpperLinLimit(std::max(lowerLin,
-        _angle.Radian()/tp));
+        _limit/tp));
     }
-    return true;
   }
   else if (_index == 1)
   {
     // _index = 1: linear constraint
     double upperLin = this->bulletScrew->getUpperLinLimit();
-    this->bulletScrew->setLowerLinLimit(std::min(upperLin, _angle.Radian()));
+    this->bulletScrew->setLowerLinLimit(std::min(upperLin, _limit));
 
     // set corresponding angular constraints
     double tp = this->threadPitch;
@@ -542,35 +531,34 @@ bool BulletScrewJoint::SetLowStop(unsigned int _index,
     {
       double upperAng = this->bulletScrew->getUpperAngLimit();
       this->bulletScrew->setLowerAngLimit(std::min(upperAng,
-        _angle.Radian()*tp));
+        _limit*tp));
     }
     else
     {
         // flip upper lower because thread pitch is negative
       double lowerAng = this->bulletScrew->getLowerAngLimit();
       this->bulletScrew->setUpperAngLimit(std::max(lowerAng,
-        _angle.Radian()*tp));
+        _limit*tp));
     }
-    return true;
   }
   else
   {
     gzerr << "Invalid index [" << _index << "]\n";
-    return false;
   }
 }
 
 //////////////////////////////////////////////////
-math::Vector3 BulletScrewJoint::GetGlobalAxis(unsigned int /*_index*/) const
+ignition::math::Vector3d BulletScrewJoint::GlobalAxis(
+    const unsigned int /*_index*/) const
 {
-  math::Vector3 result = this->initialWorldAxis;
+  ignition::math::Vector3d result = this->initialWorldAxis;
   if (this->bulletScrew)
   {
     // bullet uses x-axis for slider
     btVector3 vec =
       this->bulletScrew->getRigidBodyA().getCenterOfMassTransform().getBasis()
       * this->bulletScrew->getFrameOffsetA().getBasis().getColumn(0);
-    result = BulletTypes::ConvertVector3(vec);
+    result = BulletTypes::ConvertVector3Ign(vec);
   }
   else
     gzwarn << "bulletScrew does not exist, returning fake axis\n";
@@ -578,9 +566,9 @@ math::Vector3 BulletScrewJoint::GetGlobalAxis(unsigned int /*_index*/) const
 }
 
 //////////////////////////////////////////////////
-math::Angle BulletScrewJoint::GetAngleImpl(unsigned int _index) const
+double BulletScrewJoint::PositionImpl(const unsigned int _index) const
 {
-  math::Angle result;
+  double result = ignition::math::NAN_D;
   if (this->bulletScrew)
   {
     if (_index == 0)
@@ -601,13 +589,6 @@ math::Angle BulletScrewJoint::GetAngleImpl(unsigned int _index) const
   return result;
 }
 
-//////////////////////////////////////////////////
-double BulletScrewJoint::GetAttribute(
-  const std::string &_key, unsigned int _index)
-{
-  return this->GetParam(_key, _index);
-}
-
 //////////////////////////////////////////////////
 double BulletScrewJoint::GetParam(
   const std::string &_key, unsigned int _index)
@@ -984,7 +965,7 @@ void btScrewConstraint::_getInfo2NonVirtual(
     // if (limit && (lostop == histop)) raises warnings, using
     // a warning-less implementation.
     if (limit &&
-      math::equal(lostop, histop,
+      ignition::math::equal(lostop, histop,
       static_cast<btScalar>(std::numeric_limits<double>::epsilon())))
     {
       // the joint motor is ineffective
@@ -1020,7 +1001,7 @@ void btScrewConstraint::_getInfo2NonVirtual(
       // issue #1104:
       // if (lostop == histop) raises warnings, using
       // a warning-less implementation.
-      if (math::equal(lostop, histop,
+      if (ignition::math::equal(lostop, histop,
           static_cast<btScalar>(std::numeric_limits<double>::epsilon())))
       {
         // limited low and high simultaneously
@@ -1111,7 +1092,7 @@ void btScrewConstraint::_getInfo2NonVirtual(
     // if (limit && (lostop == histop)) raises warnings, using
     // a warning-less implementation.
     if (limit &&
-      math::equal(lostop, histop,
+      ignition::math::equal(lostop, histop,
       static_cast<btScalar>(std::numeric_limits<double>::epsilon())))
     {  // the joint motor is ineffective
       powered = 0;
@@ -1141,7 +1122,7 @@ void btScrewConstraint::_getInfo2NonVirtual(
       // issue #1104:
       // if (lostop == histop) raises warnings, using
       // a warning-less implementation.
-      if (math::equal(lostop, histop,
+      if (ignition::math::equal(lostop, histop,
           static_cast<btScalar>(std::numeric_limits<double>::epsilon())))
       {
         // limited low and high simultaneously
