@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2016 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,13 +15,8 @@
  *
 */
 
-#ifdef _WIN32
-  // Ensure that Winsock2.h is included before Windows.h, which can get
-  // pulled in by anybody (e.g., Boost).
-  #include <Winsock2.h>
-#endif
-
 #include <boost/bind.hpp>
+#include <ignition/common/Profiler.hh>
 
 #include "gazebo/common/MeshManager.hh"
 #include "gazebo/transport/transport.hh"
@@ -95,6 +90,7 @@ void LaserVisual::OnScan(ConstLaserScanStampedPtr &_msg)
 /////////////////////////////////////////////////
 void LaserVisual::Update()
 {
+  IGN_PROFILE("rendering::LaserVisual::Update");
   LaserVisualPrivate *dPtr =
       reinterpret_cast<LaserVisualPrivate *>(this->dataPtr);
 
@@ -102,8 +98,8 @@ void LaserVisual::Update()
 
   // Skip the update if the user is moving the laser.
   if ((this->GetScene()->SelectedVisual() &&
-      this->GetRootVisual()->GetName() ==
-      this->GetScene()->SelectedVisual()->GetName()))
+      this->GetRootVisual()->Name() ==
+      this->GetScene()->SelectedVisual()->Name()))
   {
     return;
   }
@@ -115,8 +111,7 @@ void LaserVisual::Update()
 
   double verticalAngle = dPtr->laserMsg->scan().vertical_angle_min();
   ignition::math::Pose3d offset =
-    msgs::ConvertIgn(dPtr->laserMsg->scan().world_pose()) -
-    this->GetWorldPose().Ign();
+    msgs::ConvertIgn(dPtr->laserMsg->scan().world_pose()) - this->WorldPose();
 
   unsigned int vertCount = dPtr->laserMsg->scan().has_vertical_count() ?
       dPtr->laserMsg->scan().vertical_count() : 1u;
@@ -133,25 +128,27 @@ void LaserVisual::Update()
       // intersected an object.
       dPtr->rayStrips.push_back(
           this->CreateDynamicLine(rendering::RENDERING_TRIANGLE_STRIP));
-      dPtr->rayStrips[j]->setMaterial("Gazebo/BlueLaser");
+      GZ_OGRE_SET_MATERIAL_BY_NAME(dPtr->rayStrips[j], "Gazebo/BlueLaser");
 
       // No hit ray strips fill in-between the ray lines in areas that have
       // not intersected an object.
       dPtr->noHitRayStrips.push_back(
           this->CreateDynamicLine(rendering::RENDERING_TRIANGLE_STRIP));
-      dPtr->noHitRayStrips[j]->setMaterial("Gazebo/LightBlueLaser");
+      GZ_OGRE_SET_MATERIAL_BY_NAME(dPtr->noHitRayStrips[j],
+          "Gazebo/LightBlueLaser");
 
       // Deadzone ray fans display areas that are between the sensor's origin
       // and start of the rays.
       dPtr->deadzoneRayFans.push_back(
           this->CreateDynamicLine(rendering::RENDERING_TRIANGLE_FAN));
-      dPtr->deadzoneRayFans[j]->setMaterial("Gazebo/BlackTransparent");
+      GZ_OGRE_SET_MATERIAL_BY_NAME(dPtr->deadzoneRayFans[j],
+          "Gazebo/BlackTransparent");
       dPtr->deadzoneRayFans[j]->AddPoint(ignition::math::Vector3d(0, 0, 0));
 
       // Individual ray lines
       dPtr->rayLines.push_back(
           this->CreateDynamicLine(rendering::RENDERING_LINE_LIST));
-      dPtr->rayLines[j]->setMaterial("Gazebo/BlueLaser");
+      GZ_OGRE_SET_MATERIAL_BY_NAME(dPtr->rayLines[j], "Gazebo/BlueLaser");
 
       this->SetVisibilityFlags(GZ_VISIBILITY_GUI);
     }
@@ -165,6 +162,11 @@ void LaserVisual::Update()
     {
       // Calculate the range of the ray
       double r = dPtr->laserMsg->scan().ranges(j*count + i);
+      if (r < minRange)
+      {
+        // Less than min range, don't display a ray
+        r = minRange;
+      }
       bool inf = std::isinf(r);
 
       ignition::math::Quaterniond ray(
@@ -225,7 +227,7 @@ void LaserVisual::Update()
 }
 
 /////////////////////////////////////////////////
-void LaserVisual::SetEmissive(const common::Color &/*_color*/,
+void LaserVisual::SetEmissive(const ignition::math::Color &/*_color*/,
     const bool /*_cascade*/)
 {
 }
