@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,6 +15,10 @@
  *
 */
 #include <boost/bind.hpp>
+#include <ignition/common/Profiler.hh>
+#include <ignition/math/Color.hh>
+#include <ignition/math/Vector2.hh>
+
 #include "gazebo/rendering/ogre_gazebo.h"
 
 #include "gazebo/common/Assert.hh"
@@ -57,7 +61,7 @@ UserCamera::UserCamera(const std::string &_name, ScenePtr _scene,
   // Set default UserCamera render rate to 120Hz when stereo rendering is
   // enabled. Otherwise use 60Hz.
   // Some padding is added for safety.
-  this->SetRenderRate(_stereoEnabled ? 130.0 : 70.0);
+  this->SetRenderRate(_stereoEnabled ? 124.0 : 62.0);
 
   this->SetUseSDFPose(false);
 }
@@ -114,14 +118,14 @@ void UserCamera::Init()
   Camera::Init();
 
   // Don't yaw along variable axis, causes leaning
-  this->camera->setFixedYawAxis(true, Ogre::Vector3::UNIT_Z);
-  this->camera->setDirection(1, 0, 0);
+  this->SetFixedYawAxis(true, ignition::math::Vector3d::UnitZ);
+  this->sceneNode->setDirection(1, 0, 0);
   this->camera->setAutoAspectRatio(false);
 
   // Right camera
   if (this->dataPtr->stereoEnabled)
   {
-    this->dataPtr->rightCamera = this->scene->GetManager()->createCamera(
+    this->dataPtr->rightCamera = this->scene->OgreSceneManager()->createCamera(
         "StereoUserRight");
     this->dataPtr->rightCamera->pitch(Ogre::Degree(90));
 
@@ -134,7 +138,7 @@ void UserCamera::Init()
     this->sceneNode->attachObject(this->dataPtr->rightCamera);
   }
 
-  this->SetHFOV(GZ_DTOR(60));
+  this->SetHFOV(static_cast<ignition::math::Angle>(IGN_DTOR(60)));
 
   // Careful when setting this value.
   // A far clip that is too close will have bad side effects on the
@@ -175,7 +179,7 @@ void UserCamera::Init()
   axisNode->setInheritOrientation(false);
 
   Ogre::ManualObject *x =
-    this->scene->GetManager()->createManualObject("MyXAxis");
+    this->scene->OgreSceneManager()->createManualObject("MyXAxis");
   x->begin("Gazebo/Red", Ogre::RenderOperation::OT_LINE_LIST);
   x->position(0, 0, 0);
   x->position(1, 0, 0);
@@ -183,7 +187,7 @@ void UserCamera::Init()
   x->setVisibilityFlags(GZ_VISIBILITY_GUI);
 
   Ogre::ManualObject *y =
-    this->scene->GetManager()->createManualObject("MyYAxis");
+    this->scene->OgreSceneManager()->createManualObject("MyYAxis");
   y->begin("Gazebo/Green", Ogre::RenderOperation::OT_LINE_LIST);
   y->position(0, 0, 0);
   y->position(0, 1, 0);
@@ -191,7 +195,7 @@ void UserCamera::Init()
   y->setVisibilityFlags(GZ_VISIBILITY_GUI);
 
   Ogre::ManualObject *z =
-    this->scene->GetManager()->createManualObject("MyZAxis");
+    this->scene->OgreSceneManager()->createManualObject("MyZAxis");
   z->begin("Gazebo/Blue", Ogre::RenderOperation::OT_LINE_LIST);
   z->position(0, 0, 0);
   z->position(0, 0, 1);
@@ -205,20 +209,20 @@ void UserCamera::Init()
 }
 
 //////////////////////////////////////////////////
-void UserCamera::SetDefaultPose(const math::Pose &_pose)
+void UserCamera::SetInitialPose(const ignition::math::Pose3d &_pose)
 {
-  this->dataPtr->defaultPose = _pose;
+  this->dataPtr->initialPose = _pose;
   this->SetWorldPose(_pose);
 }
 
 //////////////////////////////////////////////////
-math::Pose UserCamera::DefaultPose() const
+ignition::math::Pose3d UserCamera::InitialPose() const
 {
-  return this->dataPtr->defaultPose;
+  return this->dataPtr->initialPose;
 }
 
 //////////////////////////////////////////////////
-void UserCamera::SetWorldPose(const math::Pose &_pose)
+void UserCamera::SetWorldPose(const ignition::math::Pose3d &_pose)
 {
   Camera::SetWorldPose(_pose);
   this->dataPtr->viewController->Init();
@@ -227,13 +231,14 @@ void UserCamera::SetWorldPose(const math::Pose &_pose)
 //////////////////////////////////////////////////
 void UserCamera::Update()
 {
+  IGN_PROFILE("rendering::UserCamera::Update");
   Camera::Update();
 
   if (this->dataPtr->viewController)
     this->dataPtr->viewController->Update();
 
   // publish camera pose
-  this->dataPtr->posePub->Publish(msgs::Convert(this->GetWorldPose().Ign()));
+  this->dataPtr->posePub->Publish(msgs::Convert(this->WorldPose()));
 }
 
 //////////////////////////////////////////////////
@@ -242,9 +247,21 @@ void UserCamera::AnimationComplete()
   this->dataPtr->viewController->Init();
 }
 
+//////////////////////////////////////////////////
+void UserCamera::Render(const bool /*_force*/)
+{
+  IGN_PROFILE("rendering::UserCamera::Render");
+  if (this->initialized)
+  {
+    this->newData = true;
+    this->RenderImpl();
+  }
+}
+
 //////////////////////////////////////////////////
 void UserCamera::PostRender()
 {
+  IGN_PROFILE("rendering::UserCamera::PostRender");
   Camera::PostRender();
 }
 
@@ -306,31 +323,32 @@ void UserCamera::SetJoyPoseControl(bool _value)
 }
 
 /////////////////////////////////////////////////
-bool UserCamera::AttachToVisualImpl(VisualPtr _visual, bool _inheritOrientation,
-                                     double /*_minDist*/, double /*_maxDist*/)
+bool UserCamera::AttachToVisualImpl(VisualPtr _visual,
+    const bool _inheritOrientation,
+    const double /*_minDist*/, const double /*_maxDist*/)
 {
   Camera::AttachToVisualImpl(_visual, _inheritOrientation);
   if (_visual)
   {
-    math::Pose origPose = this->GetWorldPose();
-    double yaw = _visual->GetWorldPose().rot.GetAsEuler().z;
+    ignition::math::Pose3d origPose = this->WorldPose();
+    ignition::math::Angle yaw = _visual->WorldPose().Rot().Euler().Z();
 
-    double zDiff = origPose.pos.z - _visual->GetWorldPose().pos.z;
-    double pitch = 0;
+    double zDiff = origPose.Pos().Z() - _visual->WorldPose().Pos().Z();
+    ignition::math::Angle pitch = 0;
 
     if (fabs(zDiff) > 1e-3)
     {
-      double dist = _visual->GetWorldPose().pos.Distance(
-          this->GetWorldPose().pos);
+      double dist = _visual->WorldPose().Pos().Distance(
+          this->WorldPose().Pos());
       pitch = acos(zDiff/dist);
     }
 
     this->Yaw(yaw);
     this->Pitch(pitch);
 
-    math::Box bb = _visual->GetBoundingBox();
-    math::Vector3 pos = bb.GetCenter();
-    pos.z = bb.max.z;
+    auto bb = _visual->BoundingBox();
+    auto pos = bb.Center();
+    pos.Z(bb.Max().Z());
 
     this->SetViewController(OrbitViewController::GetTypeString(), pos);
   }
@@ -377,7 +395,7 @@ void UserCamera::SetViewController(const std::string &_type)
     if (vc == "orbit")
     {
       this->dataPtr->viewController->Init(
-          this->dataPtr->orbitViewController->GetFocalPoint(),
+          this->dataPtr->orbitViewController->FocalPoint(),
           this->dataPtr->orbitViewController->Yaw(),
           this->dataPtr->orbitViewController->Pitch());
     }
@@ -402,7 +420,7 @@ void UserCamera::SetViewController(const std::string &_type)
 
 //////////////////////////////////////////////////
 void UserCamera::SetViewController(const std::string &_type,
-                                   const math::Vector3 &_pos)
+                                   const ignition::math::Vector3d &_pos)
 {
   if (_type.empty() ||
       this->dataPtr->viewController->GetTypeString() == _type)
@@ -443,6 +461,32 @@ void UserCamera::Resize(unsigned int _w, unsigned int _h)
 {
   this->UpdateFOV();
   this->dataPtr->viewController->Resize(_w, _h);
+
+  // reload ogre compositors on window resize
+  // otherwise some compositors can cause the client to crash
+  Ogre::CompositorManager *compMgr =
+      Ogre::CompositorManager::getSingletonPtr();
+  if (compMgr->hasCompositorChain(this->viewport))
+  {
+    Ogre::CompositorChain *chain =
+        compMgr->getCompositorChain(this->viewport);
+    std::vector<std::pair<std::string, bool>> compositors;
+    Ogre::CompositorChain::InstanceIterator it = chain->getCompositors();
+    while (it.hasMoreElements())
+    {
+      Ogre::CompositorInstance* nextCompInst = it.getNext();
+      compositors.push_back(
+          std::make_pair(nextCompInst->getCompositor()->getName(),
+          nextCompInst->getEnabled()));
+    }
+    compMgr->removeCompositorChain(this->viewport);
+    for (unsigned int i = 0; i < compositors.size(); ++i)
+    {
+      compMgr->addCompositor(this->viewport, compositors[i].first);
+      compMgr->setCompositorEnabled(this->viewport, compositors[i].first,
+          compositors[i].second);
+    }
+  }
 }
 
 //////////////////////////////////////////////////
@@ -460,7 +504,7 @@ void UserCamera::UpdateFOV()
     double vfov = 2.0 * atan(tan(hfov / 2.0) / ratio);
 
     this->dataPtr->rightCamera->setAspectRatio(ratio);
-    this->dataPtr->rightCamera->setFOVy(Ogre::Radian(vfov));
+    this->dataPtr->rightCamera->setFOVy(Ogre::Radian(this->LimitFOV(vfov)));
   }
 }
 
@@ -483,12 +527,6 @@ void UserCamera::ShowVisual(bool /*_s*/)
   // this->visual->SetVisible(_s);
 }
 
-//////////////////////////////////////////////////
-bool UserCamera::MoveToPosition(const math::Pose &_pose, double _time)
-{
-  return Camera::MoveToPosition(_pose, _time);
-}
-
 //////////////////////////////////////////////////
 void UserCamera::MoveToVisual(const std::string &_name)
 {
@@ -505,73 +543,75 @@ void UserCamera::MoveToVisual(VisualPtr _visual)
   if (!_visual)
     return;
 
-  if (this->scene->GetManager()->hasAnimation("cameratrack"))
+  if (this->scene->OgreSceneManager()->hasAnimation("cameratrack"))
   {
-    this->scene->GetManager()->destroyAnimation("cameratrack");
+    this->scene->OgreSceneManager()->destroyAnimation("cameratrack");
   }
 
-  math::Box box = _visual->GetBoundingBox();
-  math::Vector3 size = box.GetSize();
-  double maxSize = std::max(std::max(size.x, size.y), size.z);
-
-  math::Vector3 start = this->GetWorldPose().pos;
+  // Start from current position
+  ignition::math::Vector3d start = this->WorldPose().Pos();
   start.Correct();
-  math::Vector3 end = box.GetCenter() + _visual->GetWorldPose().pos;
-  end.Correct();
-  math::Vector3 dir = end - start;
+
+  // Center of visual
+  ignition::math::AxisAlignedBox box = _visual->BoundingBox();
+  ignition::math::Vector3d visCenter = box.Center() +
+    _visual->WorldPose().Pos();
+  visCenter.Correct();
+
+  // Direction from start to visual center
+  ignition::math::Vector3d dir = visCenter - start;
   dir.Correct();
   dir.Normalize();
 
-  double dist = start.Distance(end) - maxSize;
+  // Distance to move
+  ignition::math::Vector3d size = box.Size();
+  double maxSize = size.Max();
+  double dist = start.Distance(visCenter) - maxSize;
 
-  math::Vector3 mid = start + dir*(dist*.5);
-  mid.z = box.GetCenter().z + box.GetSize().z + 2.0;
+  // Find midway point and change its Z
+  ignition::math::Vector3d mid = start + dir*(dist*.5);
+  mid.Z(box.Center().Z() + box.Size().Z() + 2.0);
 
-  dir = end - mid;
+  // Direction from mid to visual center
+  dir = visCenter - mid;
   dir.Correct();
-
-  dist = mid.Distance(end) - maxSize;
-
-  double yawAngle = atan2(dir.y, dir.x);
-  double pitchAngle = atan2(-dir.z, sqrt(dir.x*dir.x + dir.y*dir.y));
-  math::Quaternion pitchYawOnly(0, pitchAngle, yawAngle);
-  Ogre::Quaternion pitchYawFinal(pitchYawOnly.w, pitchYawOnly.x,
-    pitchYawOnly.y, pitchYawOnly.z);
-
   dir.Normalize();
 
-  double scale = maxSize / tan((this->GetHFOV()/2.0).Radian());
+  // Get new distance
+  dist = mid.Distance(visCenter) - maxSize;
 
-  end = mid + dir*(dist - scale);
+  // Scale to fit in view
+  double scale = maxSize / tan((this->HFOV()/2.0).Radian());
 
-  // dist = start.Distance(end);
-  // double vel = 5.0;
-  double time = 0.5;  // dist / vel;
+  // End position
+  auto end = mid + dir*(dist - scale);
 
+  // Orientation
+  auto mat = ignition::math::Matrix4d::LookAt(end, visCenter);
+
+  // Time
+  double time = 0.5;
+
+  // Ogre animation
   Ogre::Animation *anim =
-    this->scene->GetManager()->createAnimation("cameratrack", time);
+    this->scene->OgreSceneManager()->createAnimation("cameratrack", time);
   anim->setInterpolationMode(Ogre::Animation::IM_SPLINE);
 
   Ogre::NodeAnimationTrack *strack = anim->createNodeTrack(0, this->sceneNode);
 
-
+  // Start keyframe
   Ogre::TransformKeyFrame *key;
-
   key = strack->createNodeKeyFrame(0);
-  key->setTranslate(Ogre::Vector3(start.x, start.y, start.z));
+  key->setTranslate(Conversions::Convert(start));
   key->setRotation(this->sceneNode->getOrientation());
 
-  /*key = strack->createNodeKeyFrame(time * 0.5);
-  key->setTranslate(Ogre::Vector3(mid.x, mid.y, mid.z));
-  key->setRotation(pitchYawFinal);
-  */
-
+  // End keyframe
   key = strack->createNodeKeyFrame(time);
-  key->setTranslate(Ogre::Vector3(end.x, end.y, end.z));
-  key->setRotation(pitchYawFinal);
+  key->setTranslate(Conversions::Convert(end));
+  key->setRotation(Conversions::Convert(mat.Rotation()));
 
   this->animState =
-    this->scene->GetManager()->createAnimationState("cameratrack");
+    this->scene->OgreSceneManager()->createAnimationState("cameratrack");
 
   this->animState->setTimePosition(0);
   this->animState->setEnabled(true);
@@ -588,8 +628,35 @@ void UserCamera::MoveToVisual(VisualPtr _visual)
 void UserCamera::OnMoveToVisualComplete()
 {
   this->dataPtr->orbitViewController->SetDistance(
-      this->GetWorldPose().pos.Distance(
-      this->dataPtr->orbitViewController->GetFocalPoint()));
+      this->WorldPose().Pos().Distance(
+      this->dataPtr->orbitViewController->FocalPoint()));
+}
+
+
+//////////////////////////////////////////////////
+void UserCamera::SetDevicePixelRatio(const double _ratio)
+{
+  this->dataPtr->devicePixelRatio = _ratio;
+}
+
+//////////////////////////////////////////////////
+double UserCamera::DevicePixelRatio() const
+{
+  return this->dataPtr->devicePixelRatio;
+}
+
+
+//////////////////////////////////////////////////
+void UserCamera::CameraToViewportRay(const int _screenx,
+    const int _screeny,
+    ignition::math::Vector3d &_origin,
+    ignition::math::Vector3d &_dir) const
+{
+  int ratio = static_cast<int>(this->dataPtr->devicePixelRatio);
+  int screenx = ratio * _screenx;
+  int screeny = ratio * _screeny;
+
+  Camera::CameraToViewportRay(screenx, screeny, _origin, _dir);
 }
 
 //////////////////////////////////////////////////
@@ -613,23 +680,26 @@ void UserCamera::SetRenderTarget(Ogre::RenderTarget *_target)
 
     this->dataPtr->rightViewport =
       this->renderTarget->addViewport(this->dataPtr->rightCamera, 1);
+    auto const &ignBgColor = this->scene->BackgroundColor();
     this->dataPtr->rightViewport->setBackgroundColour(
-        Conversions::Convert(this->scene->GetBackgroundColor()));
+        Conversions::Convert(ignBgColor));
 
 #if OGRE_VERSION_MAJOR > 1 || OGRE_VERSION_MINOR > 9
     this->viewport->setDrawBuffer(Ogre::CBT_BACK_LEFT);
     this->dataPtr->rightViewport->setDrawBuffer(Ogre::CBT_BACK_RIGHT);
 #endif
 
-    this->dataPtr->rightViewport->setVisibilityMask(GZ_VISIBILITY_ALL);
+    this->dataPtr->rightViewport->setVisibilityMask(
+        GZ_VISIBILITY_ALL & ~GZ_VISIBILITY_SELECTABLE);
   }
 
-  this->viewport->setVisibilityMask(GZ_VISIBILITY_ALL);
+  this->viewport->setVisibilityMask(
+      GZ_VISIBILITY_ALL & ~GZ_VISIBILITY_SELECTABLE);
 
   this->initialized = true;
 
   this->dataPtr->selectionBuffer = new SelectionBuffer(this->scopedUniqueName,
-      this->scene->GetManager(), this->renderTarget);
+      this->scene->OgreSceneManager(), this->renderTarget);
 }
 
 //////////////////////////////////////////////////
@@ -639,16 +709,20 @@ void UserCamera::EnableViewController(bool _value) const
 }
 
 //////////////////////////////////////////////////
-VisualPtr UserCamera::GetVisual(const math::Vector2i &_mousePos,
-                                std::string &_mod)
+VisualPtr UserCamera::Visual(const ignition::math::Vector2i &_mousePos,
+    std::string &_mod) const
 {
   VisualPtr result;
 
   if (!this->dataPtr->selectionBuffer)
     return result;
 
-  Ogre::Entity *entity =
-    this->dataPtr->selectionBuffer->OnSelectionClick(_mousePos.x, _mousePos.y);
+  int ratio = static_cast<int>(this->dataPtr->devicePixelRatio);
+  ignition::math::Vector2i mousePos(
+      ratio * _mousePos.X(), ratio * _mousePos.Y());
+
+  Ogre::Entity *entity = this->dataPtr->selectionBuffer->OnSelectionClick(
+      mousePos.X(), mousePos.Y());
 
   _mod = "";
   if (entity)
@@ -691,18 +765,22 @@ VisualPtr UserCamera::GetVisual(const math::Vector2i &_mousePos,
 }
 
 //////////////////////////////////////////////////
-void UserCamera::SetFocalPoint(const math::Vector3 &_pt)
+void UserCamera::SetFocalPoint(const ignition::math::Vector3d &_pt)
 {
   this->dataPtr->orbitViewController->SetFocalPoint(_pt);
 }
 
 //////////////////////////////////////////////////
-VisualPtr UserCamera::GetVisual(const math::Vector2i &_mousePos) const
+VisualPtr UserCamera::Visual(const ignition::math::Vector2i &_mousePos) const
 {
   VisualPtr result;
 
-  Ogre::Entity *entity =
-    this->dataPtr->selectionBuffer->OnSelectionClick(_mousePos.x, _mousePos.y);
+  int ratio = static_cast<int>(this->dataPtr->devicePixelRatio);
+  ignition::math::Vector2i mousePos(
+      ratio * _mousePos.X(), ratio * _mousePos.Y());
+
+  Ogre::Entity *entity = this->dataPtr->selectionBuffer->OnSelectionClick(
+      mousePos.X(), mousePos.Y());
 
   if (entity && !entity->getUserObjectBindings().getUserAny().isEmpty())
   {
@@ -752,7 +830,7 @@ void UserCamera::OnJoyTwist(ConstJoystickPtr &_msg)
   if (this->dataPtr->joyTwistControl &&
       (_msg->has_translation() || _msg->has_rotation()))
   {
-    math::Pose pose = this->GetWorldPose();
+    ignition::math::Pose3d pose = this->WorldPose();
 
     // Get the joystick XYZ
     if (_msg->has_translation())
@@ -760,7 +838,7 @@ void UserCamera::OnJoyTwist(ConstJoystickPtr &_msg)
       const double transRotRatio = 0.05;
       ignition::math::Vector3d trans =
         msgs::ConvertIgn(_msg->translation()) * transRotRatio;
-      pose.pos = pose.rot.RotateVector(trans) + pose.pos;
+      pose.Pos() = pose.Rot().RotateVector(trans) + pose.Pos();
     }
 
     // Get the jostick RPY. We are disabling rotation around x.
@@ -768,7 +846,7 @@ void UserCamera::OnJoyTwist(ConstJoystickPtr &_msg)
     {
       ignition::math::Vector3d rot =
         msgs::ConvertIgn(_msg->rotation()) * rpyFactor;
-      pose.rot.SetFromEuler(pose.rot.GetAsEuler() + rot);
+      pose.Rot().Euler(pose.Rot().Euler() + rot);
     }
 
     this->SetWorldPose(pose);
@@ -791,7 +869,7 @@ void UserCamera::OnJoyPose(ConstPosePtr &_msg)
 }
 
 //////////////////////////////////////////////////
-void UserCamera::SetClipDist(float _near, float _far)
+void UserCamera::SetClipDist(const float _near, const float _far)
 {
   Camera::SetClipDist(_near, _far);
 
@@ -854,3 +932,11 @@ bool UserCamera::SetProjectionType(const std::string &_type)
 
   return Camera::SetProjectionType(_type);
 }
+
+/////////////////////////////////////////////////
+ignition::math::Vector2i UserCamera::Project(
+    const ignition::math::Vector3d &_pt) const
+{
+  auto pt = Camera::Project(_pt);
+  return pt / this->dataPtr->devicePixelRatio;
+}
