@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2016 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,24 +14,26 @@
  * limitations under the License.
  *
 */
-#ifdef _WIN32
-  // Ensure that Winsock2.h is included before Windows.h, which can get
-  // pulled in by anybody (e.g., Boost).
-  #include <Winsock2.h>
-#endif
 
 #include <functional>
 #include <boost/bind.hpp>
-#include "gazebo/common/Assert.hh"
-#include "gazebo/common/Time.hh"
 
+#include "gazebo/physics/Link.hh"
+#include "gazebo/physics/Model.hh"
 #include "gazebo/physics/PhysicsIface.hh"
 #include "gazebo/physics/PhysicsEngine.hh"
+#include "gazebo/physics/PhysicsIface.hh"
 #include "gazebo/physics/World.hh"
+#include "gazebo/rendering/Camera.hh"
+#include "gazebo/sensors/CameraSensor.hh"
 #include "gazebo/sensors/Sensor.hh"
-#include "gazebo/sensors/SensorsIface.hh"
 #include "gazebo/sensors/SensorFactory.hh"
 #include "gazebo/sensors/SensorManager.hh"
+#include "gazebo/sensors/SensorsIface.hh"
+#include "gazebo/transport/transport.hh"
+#include "gazebo/util/LogPlay.hh"
+
+#include "ignition/common/Profiler.hh"
 
 using namespace gazebo;
 using namespace sensors;
@@ -40,6 +42,50 @@ using namespace sensors;
 /// for timing coordination.
 boost::mutex g_sensorTimingMutex;
 
+/// \brief Flag to indicate if number of sensors has changed and that the
+/// max update rate needs to be recalculated
+bool g_sensorsDirty = true;
+
+/// Performance metrics variables
+/// \brief last sensor measurement sim time
+std::map<std::string, gazebo::common::Time> sensorsLastMeasurementTime;
+
+/// \brief last sensor measurement real time
+std::map<std::string, gazebo::common::Time> worldLastMeasurementTime;
+
+/// \brief Data structure for storing metric data to be published
+struct sensorPerformanceMetricsType
+{
+  /// \brief sensor real time update rate
+  double sensorRealUpdateRate;
+
+  /// \brief sensor sim time update rate
+  double sensorSimUpdateRate;
+
+  /// \brief Rendering sensor average real time update rate. The difference
+  /// between sensorAvgFps and sensorRealUpdateRte is that sensorAvgFps is
+  /// for rendering sensors only and the rate is averaged over a fixed
+  /// window size, whereas the sensorSimUpdateRate stores the instantaneous
+  /// update rate and it is filled by all sensors.
+  double sensorAvgFPS;
+};
+
+/// \brief A map of sensor name to its performance metrics data
+std::map<std::string, struct sensorPerformanceMetricsType>
+    sensorPerformanceMetrics;
+
+/// \brief Publisher for run-time simulation performance metrics.
+transport::PublisherPtr performanceMetricsPub;
+
+/// \brief Node for publishing performance metrics
+transport::NodePtr node = nullptr;
+
+/// \brief Last sim time measured for performance metrics
+common::Time lastSimTime;
+
+/// \brief Last real time measured for performance metrics
+common::Time lastRealTime;
+
 //////////////////////////////////////////////////
 SensorManager::SensorManager()
   : initialized(false), removeAllSensors(false)
@@ -57,11 +103,12 @@ SensorManager::SensorManager()
 //////////////////////////////////////////////////
 SensorManager::~SensorManager()
 {
+  sensors::disable();
   // Clean up the sensors.
   for (SensorContainer_V::iterator iter = this->sensorContainers.begin();
        iter != this->sensorContainers.end(); ++iter)
   {
-    GZ_ASSERT((*iter) != NULL, "Sensor Constainer is NULL");
+    GZ_ASSERT((*iter) != nullptr, "Sensor Constainer is null");
     (*iter)->Stop();
     (*iter)->RemoveSensors();
     delete (*iter);
@@ -81,7 +128,7 @@ void SensorManager::RunThreads()
   for (SensorContainer_V::iterator iter = ++this->sensorContainers.begin();
        iter != this->sensorContainers.end(); ++iter)
   {
-    GZ_ASSERT((*iter) != NULL, "Sensor Constainer is NULL");
+    GZ_ASSERT((*iter) != nullptr, "Sensor Constainer is null");
     (*iter)->Run();
   }
 }
@@ -93,9 +140,157 @@ void SensorManager::Stop()
   for (SensorContainer_V::iterator iter = this->sensorContainers.begin();
        iter != this->sensorContainers.end(); ++iter)
   {
-    GZ_ASSERT((*iter) != NULL, "Sensor Constainer is NULL");
+    GZ_ASSERT((*iter) != nullptr, "Sensor Constainer is null");
     (*iter)->Stop();
   }
+
+  if (!physics::worlds_running())
+    this->worlds.clear();
+}
+
+//////////////////////////////////////////////////
+bool SensorManager::Running() const
+{
+  for (auto const &container : this->sensorContainers)
+  {
+    if (container->Running())
+      return true;
+  }
+  return false;
+}
+
+//////////////////////////////////////////////////
+void SensorManager::WaitForSensors(double _clk, double _dt)
+{
+  double tnext = this->NextRequiredTimestamp();
+
+  while (!std::isnan(tnext)
+      && ignition::math::lessOrNearEqual(tnext - _dt / 2.0, _clk)
+      && physics::worlds_running())
+  {
+    this->WaitForPrerendered(0.001);
+    tnext = this->NextRequiredTimestamp();
+  }
+}
+
+void PublishPerformanceMetrics()
+{
+  if (node == nullptr)
+  {
+    auto world = physics::get_world();
+    // Transport
+    node = transport::NodePtr(new transport::Node());
+    node->Init(world->Name());
+    performanceMetricsPub =
+     node->Advertise<msgs::PerformanceMetrics>(
+         "/gazebo/performance_metrics", 10, 5);
+  }
+
+  if (!performanceMetricsPub || !performanceMetricsPub->HasConnections())
+  {
+    return;
+  }
+
+  physics::WorldPtr world = physics::get_world(
+    gazebo::physics::get_world()->Name());
+
+  /// Outgoing run-time simulation performance metrics.
+  msgs::PerformanceMetrics performanceMetricsMsg;
+
+  // Real time factor
+  common::Time realTime = world->RealTime();
+  common::Time diffRealtime = realTime - lastRealTime;
+  common::Time simTime = world->SimTime();
+  common::Time diffSimTime = simTime - lastSimTime;
+  common::Time realTimeFactor;
+
+  if (ignition::math::equal(diffSimTime.Double(), 0.0))
+    return;
+
+  if (realTime == 0)
+    realTimeFactor = 0;
+  else
+    realTimeFactor = diffSimTime / diffRealtime;
+
+  performanceMetricsMsg.set_real_time_factor(realTimeFactor.Double());
+
+  lastRealTime = realTime;
+  lastSimTime = simTime;
+
+  /// update sim time for sensors
+  for (auto model: world->Models())
+  {
+    for (auto link: model->GetLinks())
+    {
+      for (unsigned int i = 0; i < link->GetSensorCount(); i++)
+      {
+        std::string name = link->GetSensorName(i);
+        sensors::SensorPtr sensor = sensors::get_sensor(name);
+
+        auto ret = sensorsLastMeasurementTime.insert(
+            std::pair<std::string, gazebo::common::Time>(name, 0));
+        worldLastMeasurementTime.insert(
+                std::pair<std::string, gazebo::common::Time>(name, 0));
+        if (ret.second == false)
+        {
+          double updateSimRate =
+            (sensor->LastMeasurementTime() - sensorsLastMeasurementTime[name])
+            .Double();
+          if (updateSimRate > 0.0)
+          {
+            sensorsLastMeasurementTime[name] = sensor->LastMeasurementTime();
+            double updateRealRate =
+              (world->RealTime() - worldLastMeasurementTime[name]).Double();
+
+            struct sensorPerformanceMetricsType emptySensorPerfomanceMetrics;
+            auto ret2 = sensorPerformanceMetrics.insert(
+                std::pair<std::string, struct sensorPerformanceMetricsType>
+                  (name, emptySensorPerfomanceMetrics));
+            if (ret2.second == false)
+            {
+              ret2.first->second.sensorSimUpdateRate =
+                  1.0/updateSimRate;
+              ret2.first->second.sensorRealUpdateRate =
+                  1.0/updateRealRate;
+              worldLastMeasurementTime[name] = world->RealTime();
+
+              // Special case for stereo cameras
+              sensors::CameraSensorPtr cameraSensor =
+                std::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
+              if (nullptr != cameraSensor)
+              {
+                ret2.first->second.sensorAvgFPS =
+                    cameraSensor->Camera()->AvgFPS();
+              }
+              else
+              {
+                ret2.first->second.sensorAvgFPS = -1;
+              }
+            }
+          }
+        }
+      }
+    }
+  }
+
+  for (auto sensorPerformanceMetric: sensorPerformanceMetrics)
+  {
+    msgs::PerformanceMetrics::PerformanceSensorMetrics
+        *performanceSensorMetricsMsg = performanceMetricsMsg.add_sensor();
+    performanceSensorMetricsMsg->set_name(sensorPerformanceMetric.first);
+    performanceSensorMetricsMsg->set_real_update_rate(
+      sensorPerformanceMetric.second.sensorRealUpdateRate);
+    performanceSensorMetricsMsg->set_sim_update_rate(
+      sensorPerformanceMetric.second.sensorSimUpdateRate);
+    if (sensorPerformanceMetric.second.sensorAvgFPS >= 0.0)
+    {
+      performanceSensorMetricsMsg->set_fps(
+        sensorPerformanceMetric.second.sensorAvgFPS);
+    }
+  }
+
+  // Publish data
+  performanceMetricsPub->Publish(performanceMetricsMsg);
 }
 
 //////////////////////////////////////////////////
@@ -107,7 +302,7 @@ void SensorManager::Update(bool _force)
     if (this->worlds.empty() && physics::worlds_running() && this->initialized)
     {
       auto world = physics::get_world();
-      this->worlds[world->GetName()] = world;
+      this->worlds[world->Name()] = world;
       world->_SetSensorsInitialized(true);
     }
 
@@ -116,11 +311,11 @@ void SensorManager::Update(bool _force)
       // in case things are spawned, sensors length changes
       for (auto &sensor : this->initSensors)
       {
-        GZ_ASSERT(sensor != NULL, "Sensor pointer is NULL");
+        GZ_ASSERT(sensor != nullptr, "Sensor pointer is null");
         GZ_ASSERT(sensor->Category() < 0 ||
             sensor->Category() < CATEGORY_COUNT, "Sensor category is empty");
-        GZ_ASSERT(this->sensorContainers[sensor->Category()] != NULL,
-            "Sensor container is NULL");
+        GZ_ASSERT(this->sensorContainers[sensor->Category()] != nullptr,
+            "Sensor container is null");
 
         sensor->Init();
         this->sensorContainers[sensor->Category()]->AddSensor(sensor);
@@ -139,7 +334,7 @@ void SensorManager::Update(bool _force)
       for (SensorContainer_V::iterator iter2 = this->sensorContainers.begin();
            iter2 != this->sensorContainers.end() && !removed; ++iter2)
       {
-        GZ_ASSERT((*iter2) != NULL, "SensorContainer is NULL");
+        GZ_ASSERT((*iter2) != nullptr, "SensorContainer is null");
 
         removed = (*iter2)->RemoveSensor(*iter);
       }
@@ -158,10 +353,14 @@ void SensorManager::Update(bool _force)
       for (SensorContainer_V::iterator iter2 = this->sensorContainers.begin();
           iter2 != this->sensorContainers.end(); ++iter2)
       {
-        GZ_ASSERT((*iter2) != NULL, "SensorContainer is NULL");
+        GZ_ASSERT((*iter2) != nullptr, "SensorContainer is null");
         (*iter2)->RemoveSensors();
       }
       this->initSensors.clear();
+
+      // Also clear the list of worlds
+      this->worlds.clear();
+
       this->removeAllSensors = false;
     }
   }
@@ -169,6 +368,8 @@ void SensorManager::Update(bool _force)
   // Only update if there are sensors
   if (this->sensorContainers[sensors::IMAGE]->sensors.size() > 0)
     this->sensorContainers[sensors::IMAGE]->Update(_force);
+
+  PublishPerformanceMetrics();
 }
 
 //////////////////////////////////////////////////
@@ -186,11 +387,32 @@ void SensorManager::ResetLastUpdateTimes()
   for (SensorContainer_V::iterator iter = this->sensorContainers.begin();
        iter != this->sensorContainers.end(); ++iter)
   {
-    GZ_ASSERT((*iter) != NULL, "SensorContainer is NULL");
+    GZ_ASSERT((*iter) != nullptr, "SensorContainer is null");
     (*iter)->ResetLastUpdateTimes();
   }
 }
 
+//////////////////////////////////////////////////
+double SensorManager::NextRequiredTimestamp()
+{
+  double rv = std::numeric_limits<double>::quiet_NaN();
+
+  // scan all sensors whose category is IMAGE
+  for (auto& s : this->sensorContainers[sensors::IMAGE]->sensors)
+  {
+    // skip deactivated sensors
+    if (!s->IsActive()) continue;
+
+    double candidate = s->NextRequiredTimestamp();
+    // take the smallest valid value
+    if (!std::isnan(candidate)
+        && (std::isnan(rv) || rv > candidate))
+      rv = candidate;
+  }
+
+  return rv;
+}
+
 //////////////////////////////////////////////////
 void SensorManager::Init()
 {
@@ -202,7 +424,7 @@ void SensorManager::Init()
   for (SensorContainer_V::iterator iter = this->sensorContainers.begin();
        iter != this->sensorContainers.end(); ++iter)
   {
-    GZ_ASSERT((*iter) != NULL, "SensorContainer is NULL");
+    GZ_ASSERT((*iter) != nullptr, "SensorContainer is null");
     (*iter)->Init();
   }
 
@@ -232,7 +454,7 @@ void SensorManager::Fini()
   for (SensorContainer_V::iterator iter = this->sensorContainers.begin();
        iter != this->sensorContainers.end(); ++iter)
   {
-    GZ_ASSERT((*iter) != NULL, "SensorContainer is NULL");
+    GZ_ASSERT((*iter) != nullptr, "SensorContainer is null");
     (*iter)->Fini();
     (*iter)->Stop();
   }
@@ -242,7 +464,7 @@ void SensorManager::Fini()
   this->worlds.clear();
 
   delete this->simTimeEventHandler;
-  this->simTimeEventHandler = NULL;
+  this->simTimeEventHandler = nullptr;
 
   this->initialized = false;
 }
@@ -284,6 +506,12 @@ std::string SensorManager::CreateSensor(sdf::ElementPtr _elem,
   sensor->Load(_worldName, _elem);
   this->worlds[_worldName] = physics::get_world(_worldName);
 
+  // Provide the wait function to the given world
+  if (sensor->StrictRate())
+    this->worlds[_worldName]->SetSensorWaitFunc(
+        std::bind(&SensorManager::WaitForSensors, this,
+          std::placeholders::_1, std::placeholders::_2));
+
   // If the SensorManager has not been initialized, then it's okay to push
   // the sensor into one of the sensor vectors because the sensor will get
   // initialized in SensorManager::Init
@@ -315,7 +543,7 @@ SensorPtr SensorManager::GetSensor(const std::string &_name) const
   for (iter = this->sensorContainers.begin();
        iter != this->sensorContainers.end() && !result; ++iter)
   {
-    GZ_ASSERT((*iter) != NULL, "SensorContainer is NULL");
+    GZ_ASSERT((*iter) != nullptr, "SensorContainer is null");
     result = (*iter)->GetSensor(_name);
   }
 
@@ -329,7 +557,7 @@ SensorPtr SensorManager::GetSensor(const std::string &_name) const
     for (iter = this->sensorContainers.begin();
          iter != this->sensorContainers.end(); ++iter)
     {
-      GZ_ASSERT((*iter) != NULL, "SensorContainer is NULL");
+      GZ_ASSERT((*iter) != nullptr, "SensorContainer is null");
       tmpSensor = (*iter)->GetSensor(_name, true);
 
       if (!tmpSensor)
@@ -338,7 +566,8 @@ SensorPtr SensorManager::GetSensor(const std::string &_name) const
       if (!result)
       {
         result = tmpSensor;
-        GZ_ASSERT(result != NULL, "SensorContainer contains a NULL Sensor");
+        GZ_ASSERT(result != nullptr,
+            "SensorContainer contains a nullptr Sensor");
       }
       else
       {
@@ -364,7 +593,7 @@ Sensor_V SensorManager::GetSensors() const
   for (SensorContainer_V::const_iterator iter = this->sensorContainers.begin();
        iter != this->sensorContainers.end(); ++iter)
   {
-    GZ_ASSERT((*iter) != NULL, "SensorContainer is NULL");
+    GZ_ASSERT((*iter) != nullptr, "SensorContainer is null");
     std::copy((*iter)->sensors.begin(), (*iter)->sensors.end(),
               std::back_inserter(result));
   }
@@ -391,6 +620,16 @@ void SensorManager::RemoveSensor(const std::string &_name)
   }
 }
 
+//////////////////////////////////////////////////
+bool SensorManager::WaitForPrerendered(double _timeoutsec)
+{
+  if (this->sensorContainers[sensors::IMAGE]->sensors.size() > 0)
+    return ((ImageSensorContainer*)this->sensorContainers[sensors::IMAGE])
+                ->WaitForPrerendered(_timeoutsec);
+
+  return true;
+}
+
 //////////////////////////////////////////////////
 void SensorManager::RemoveSensors()
 {
@@ -403,7 +642,7 @@ SensorManager::SensorContainer::SensorContainer()
 {
   this->stop = true;
   this->initialized = false;
-  this->runThread = NULL;
+  this->runThread = nullptr;
 }
 
 //////////////////////////////////////////////////
@@ -419,7 +658,7 @@ void SensorManager::SensorContainer::Init()
 
   for (auto &sensor : this->sensors)
   {
-    GZ_ASSERT(sensor != NULL, "Sensor is NULL");
+    GZ_ASSERT(sensor != nullptr, "Sensor is null");
     sensor->Init();
   }
 
@@ -436,7 +675,7 @@ void SensorManager::SensorContainer::Fini()
   // Finialize each sensor in the current sensor vector
   for (iter = this->sensors.begin(); iter != this->sensors.end(); ++iter)
   {
-    GZ_ASSERT((*iter) != NULL, "Sensor is NULL");
+    GZ_ASSERT((*iter) != nullptr, "Sensor is null");
     (*iter)->Fini();
   }
 
@@ -467,20 +706,26 @@ void SensorManager::SensorContainer::Stop()
     // this->runThread->interrupt();
     this->runThread->join();
     delete this->runThread;
-    this->runThread = NULL;
+    this->runThread = nullptr;
   }
 }
 
+//////////////////////////////////////////////////
+bool SensorManager::SensorContainer::Running() const
+{
+  return !this->stop;
+}
+
 //////////////////////////////////////////////////
 void SensorManager::SensorContainer::RunLoop()
 {
   this->stop = false;
 
   physics::WorldPtr world = physics::get_world();
-  GZ_ASSERT(world != NULL, "Pointer to World is NULL");
+  GZ_ASSERT(world != nullptr, "Pointer to World is null");
 
-  physics::PhysicsEnginePtr engine = world->GetPhysicsEngine();
-  GZ_ASSERT(engine != NULL, "Pointer to PhysicsEngine is NULL");
+  physics::PhysicsEnginePtr engine = world->Physics();
+  GZ_ASSERT(engine != nullptr, "Pointer to PhysicsEngine is null");
 
   engine->InitForThread();
 
@@ -489,6 +734,9 @@ void SensorManager::SensorContainer::RunLoop()
   // large step size.
   double maxSensorUpdate = engine->GetMaxStepSize() * 1000;
 
+  // Release engine pointer, we don't need it in the loop
+  engine.reset();
+
   common::Time sleepTime, startTime, eventTime, diffTime;
   double maxUpdateRate = 0;
 
@@ -504,26 +752,41 @@ void SensorManager::SensorContainer::RunLoop()
       return;
   }
 
+
+  auto computeMaxUpdateRate = [&]()
   {
-    boost::recursive_mutex::scoped_lock lock(this->mutex);
-
-    // Get the minimum update rate from the sensors.
-    for (Sensor_V::iterator iter = this->sensors.begin();
-        iter != this->sensors.end() && !this->stop; ++iter)
     {
-      GZ_ASSERT((*iter) != NULL, "Sensor is NULL");
-      maxUpdateRate = std::max((*iter)->UpdateRate(), maxUpdateRate);
-    }
-  }
+      boost::recursive_mutex::scoped_lock lock(this->mutex);
 
-  // Calculate an appropriate sleep time.
-  if (maxUpdateRate > 0)
-    sleepTime.Set(1.0 / (maxUpdateRate));
-  else
-    sleepTime.Set(0, 1e6);
+      if (!g_sensorsDirty)
+        return;
+
+      // Get the minimum update rate from the sensors.
+      for (Sensor_V::iterator iter = this->sensors.begin();
+          iter != this->sensors.end() && !this->stop; ++iter)
+      {
+        GZ_ASSERT((*iter) != nullptr, "Sensor is null");
+        maxUpdateRate = std::max((*iter)->UpdateRate(), maxUpdateRate);
+      }
+
+      g_sensorsDirty = false;
+    }
+
+    // Calculate an appropriate sleep time.
+    if (maxUpdateRate > 0)
+      sleepTime.Set(1.0 / (maxUpdateRate));
+    else
+      sleepTime.Set(0, 1e6);
+  };
+
+  computeMaxUpdateRate();
+
+  IGN_PROFILE_THREAD_NAME("SensorManager");
 
   while (!this->stop)
   {
+    IGN_PROFILE("SensorManager::RunLoop");
+
     // If all the sensors get deleted, wait here.
     // Use a while loop since world resets will notify the runCondition.
     while (this->sensors.empty())
@@ -533,26 +796,39 @@ void SensorManager::SensorContainer::RunLoop()
         return;
     }
 
-    // Get the start time of the update.
-    startTime = world->GetSimTime();
+    computeMaxUpdateRate();
 
+    // Get the start time of the update.
+    startTime = world->SimTime();
+
+    IGN_PROFILE_BEGIN("UpdateSensors");
     this->Update(false);
+    IGN_PROFILE_END();
 
     // Compute the time it took to update the sensors.
     // It's possible that the world time was reset during the Update. This
     // would case a negative diffTime. Instead, just use a event time of zero
-    diffTime = std::max(common::Time::Zero, world->GetSimTime() - startTime);
+    diffTime = std::max(common::Time::Zero, world->SimTime() - startTime);
 
     // Set the default sleep time
     eventTime = std::max(common::Time::Zero, sleepTime - diffTime);
 
     // Make sure update time is reasonable.
-    GZ_ASSERT(diffTime.sec < maxSensorUpdate,
-        "Took over 1000*max_step_size to update a sensor.");
+    // During log playback, time can jump forward an arbitrary amount.
+    if (diffTime.sec >= maxSensorUpdate && !util::LogPlay::Instance()->IsOpen())
+    {
+      gzwarn << "Took over 1000*max_step_size to update a sensor "
+        << "(took " << diffTime.sec << " sec, which is more than "
+        << "the max update of " << maxSensorUpdate << " sec). "
+        << "This warning can be ignored during log playback" << std::endl;
+    }
 
     // Make sure eventTime is not negative.
-    GZ_ASSERT(eventTime >= common::Time::Zero,
-        "Time to next sensor update is negative.");
+    if (eventTime < common::Time::Zero)
+    {
+      gzerr << "Time to next sensor update is negative." << std::endl;
+      continue;
+    }
 
     boost::mutex::scoped_lock timingLock(g_sensorTimingMutex);
 
@@ -562,10 +838,12 @@ void SensorManager::SensorContainer::RunLoop()
         eventTime, &this->runCondition);
 
     // This if statement helps prevent deadlock on osx during teardown.
+    IGN_PROFILE_BEGIN("Sleeping");
     if (!this->stop)
     {
       this->runCondition.wait(timingLock);
     }
+    IGN_PROFILE_END();
   }
 }
 
@@ -581,8 +859,10 @@ void SensorManager::SensorContainer::Update(bool _force)
   for (Sensor_V::iterator iter = this->sensors.begin();
        iter != this->sensors.end(); ++iter)
   {
-    GZ_ASSERT((*iter) != NULL, "Sensor is NULL");
+    GZ_ASSERT((*iter) != nullptr, "Sensor is null");
+    IGN_PROFILE_BEGIN((*iter)->Name().c_str());
     (*iter)->Update(_force);
+    IGN_PROFILE_END();
   }
 }
 
@@ -598,7 +878,7 @@ SensorPtr SensorManager::SensorContainer::GetSensor(const std::string &_name,
   for (Sensor_V::const_iterator iter = this->sensors.begin();
        iter != this->sensors.end() && !result; ++iter)
   {
-    GZ_ASSERT((*iter) != NULL, "Sensor is NULL");
+    GZ_ASSERT((*iter) != nullptr, "Sensor is null");
 
     // We match on the scoped name (model::link::sensor) because multiple
     // sensors with the same leaf name may exist in a world.
@@ -616,11 +896,12 @@ SensorPtr SensorManager::SensorContainer::GetSensor(const std::string &_name,
 //////////////////////////////////////////////////
 void SensorManager::SensorContainer::AddSensor(SensorPtr _sensor)
 {
-  GZ_ASSERT(_sensor != NULL, "Sensor is NULL when passed to ::AddSensor");
+  GZ_ASSERT(_sensor != nullptr, "Sensor is nullptr when passed to ::AddSensor");
 
   {
     boost::recursive_mutex::scoped_lock lock(this->mutex);
     this->sensors.push_back(_sensor);
+    g_sensorsDirty = true;
   }
 
   // Tell the run loop that we have received a sensor
@@ -639,7 +920,7 @@ bool SensorManager::SensorContainer::RemoveSensor(const std::string &_name)
   // Find the correct sensor based on name, and remove it.
   for (iter = this->sensors.begin(); iter != this->sensors.end(); ++iter)
   {
-    GZ_ASSERT((*iter) != NULL, "Sensor is NULL");
+    GZ_ASSERT((*iter) != nullptr, "Sensor is null");
 
     if ((*iter)->ScopedName() == _name)
     {
@@ -650,6 +931,8 @@ bool SensorManager::SensorContainer::RemoveSensor(const std::string &_name)
     }
   }
 
+  g_sensorsDirty = true;
+
   return removed;
 }
 
@@ -663,7 +946,7 @@ void SensorManager::SensorContainer::ResetLastUpdateTimes()
   // Rest last update times for all contained sensors.
   for (iter = this->sensors.begin(); iter != this->sensors.end(); ++iter)
   {
-    GZ_ASSERT((*iter) != NULL, "Sensor is NULL");
+    GZ_ASSERT((*iter) != nullptr, "Sensor is null");
     (*iter)->ResetLastUpdateTime();
   }
 
@@ -681,18 +964,27 @@ void SensorManager::SensorContainer::RemoveSensors()
   // Remove all the sensors
   for (iter = this->sensors.begin(); iter != this->sensors.end(); ++iter)
   {
-    GZ_ASSERT((*iter) != NULL, "Sensor is NULL");
+    GZ_ASSERT((*iter) != nullptr, "Sensor is null");
     (*iter)->Fini();
   }
 
+  g_sensorsDirty = true;
+
   this->sensors.clear();
 }
 
 //////////////////////////////////////////////////
 void SensorManager::ImageSensorContainer::Update(bool _force)
 {
+  // Prerender phase
   event::Events::preRender();
 
+  // Signals end of prerender phase
+  event::Events::preRenderEnded();
+
+  // Notify that prerender is over
+  this->conditionPrerendered.notify_all();
+
   // Tell all the cameras to render
   event::Events::render();
 
@@ -702,8 +994,16 @@ void SensorManager::ImageSensorContainer::Update(bool _force)
   SensorContainer::Update(_force);
 }
 
-
-
+//////////////////////////////////////////////////
+bool SensorManager::ImageSensorContainer::WaitForPrerendered(double _timeoutsec)
+{
+  std::cv_status ret;
+  std::mutex mtx;
+  std::unique_lock<std::mutex> lck(mtx);
+  ret = this->conditionPrerendered.wait_for(lck,
+      std::chrono::duration<double>(_timeoutsec));
+  return (ret == std::cv_status::no_timeout);
+}
 
 /////////////////////////////////////////////////
 SimTimeEventHandler::SimTimeEventHandler()
@@ -719,7 +1019,7 @@ SimTimeEventHandler::~SimTimeEventHandler()
   for (std::list<SimTimeEvent*>::iterator iter = this->events.begin();
        iter != this->events.end(); ++iter)
   {
-    GZ_ASSERT(*iter != NULL, "SimTimeEvent is NULL");
+    GZ_ASSERT(*iter != nullptr, "SimTimeEvent is null");
     delete *iter;
   }
   this->events.clear();
@@ -732,11 +1032,11 @@ void SimTimeEventHandler::AddRelativeEvent(const common::Time &_time,
   boost::mutex::scoped_lock lock(this->mutex);
 
   physics::WorldPtr world = physics::get_world();
-  GZ_ASSERT(world != NULL, "World pointer is NULL");
+  GZ_ASSERT(world != nullptr, "World pointer is null");
 
   // Create the new event.
   SimTimeEvent *event = new SimTimeEvent;
-  event->time = world->GetSimTime() + _time;
+  event->time = world->SimTime() + _time;
   event->condition = _var;
 
   // Add the event to the list.
@@ -753,7 +1053,7 @@ void SimTimeEventHandler::OnUpdate(const common::UpdateInfo &_info)
   for (std::list<SimTimeEvent*>::iterator iter = this->events.begin();
       iter != this->events.end();)
   {
-    GZ_ASSERT(*iter != NULL, "SimTimeEvent is NULL");
+    GZ_ASSERT(*iter != nullptr, "SimTimeEvent is null");
 
     // Find events that have a time less than or equal to simulation
     // time.
