@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2016 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,16 +14,13 @@
  * limitations under the License.
  *
 */
-#ifdef _WIN32
-  // Ensure that Winsock2.h is included before Windows.h, which can get
-  // pulled in by anybody (e.g., Boost).
-#include <Winsock2.h>
-#endif
-
 #include <boost/algorithm/string.hpp>
+#include <regex>
 #include <stdio.h>
 #include <string>
 #include <cmath>
+#include <functional>
+#include <ignition/math/Helpers.hh>
 
 #include "gazebo/gazebo.hh"
 #include "ServerFixture.hh"
@@ -167,7 +164,10 @@ void ServerFixture::Load(const std::string &_worldFilename,
                          bool _paused, const std::string &_physics,
                          const std::vector<std::string> &_systemPlugins)
 {
-  std::string params = _worldFilename;
+  // Substitute spaces in file name with a placeholder to prevent the name
+  // from being split later
+  auto params = std::regex_replace(_worldFilename, std::regex("\\s"), "%20");
+
   if (!_physics.empty())
     params += " -e " + _physics;
   if (_paused)
@@ -190,13 +190,17 @@ void ServerFixture::LoadArgs(const std::string &_args)
   boost::trim_if(args, boost::is_any_of("\t "));
   boost::split(params, args, boost::is_any_of("\t "), boost::token_compress_on);
 
+  // Add spaces back
+  for (auto &param : params)
+    param = std::regex_replace(param, std::regex("%20"), " ");
+
   bool paused = false;
   if (std::find(params.begin(), params.end(), "-u") != params.end())
     paused = true;
 
   // Create, load, and run the server in its own thread
   this->serverThread = new boost::thread(
-    boost::bind(&ServerFixture::RunServer, this, params));
+    std::bind(&ServerFixture::RunServer, this, params));
 
   // Wait for the server to come up
   // Use a 60 second timeout.
@@ -263,23 +267,19 @@ void ServerFixture::RunServer(const std::vector<std::string> &_args)
 
   ASSERT_NO_THROW(this->server = new Server());
 
-  if (!this->server->ParseArgs(argc, argv))
+  if (this->server->ParseArgs(argc, argv))
   {
-    ASSERT_NO_THROW(delete this->server);
-    this->server = NULL;
-    return;
+    if (!rendering::get_scene(gazebo::physics::get_world()->Name()))
+    {
+      ASSERT_NO_THROW(rendering::create_scene(
+            gazebo::physics::get_world()->Name(), false, true));
+    }
+
+    ASSERT_NO_THROW(this->server->Run());
+
+    ASSERT_NO_THROW(this->server->Fini());
   }
 
-  if (!rendering::get_scene(gazebo::physics::get_world()->GetName()))
-  {
-    ASSERT_NO_THROW(rendering::create_scene(
-        gazebo::physics::get_world()->GetName(), false, true));
-  }
-
-  ASSERT_NO_THROW(this->server->Run());
-
-  ASSERT_NO_THROW(this->server->Fini());
-
   ASSERT_NO_THROW(delete this->server);
   this->server = NULL;
 
@@ -350,7 +350,7 @@ double ServerFixture::GetPercentRealTime() const
 /////////////////////////////////////////////////
 void ServerFixture::OnPose(ConstPosesStampedPtr &_msg)
 {
-  boost::mutex::scoped_lock lock(this->receiveMutex);
+  std::lock_guard<std::mutex> lock(this->receiveMutex);
   for (int i = 0; i < _msg->pose_size(); ++i)
   {
     this->poses[_msg->pose(i).name()] = msgs::ConvertIgn(_msg->pose(i));
@@ -358,11 +358,12 @@ void ServerFixture::OnPose(ConstPosesStampedPtr &_msg)
 }
 
 /////////////////////////////////////////////////
-math::Pose ServerFixture::GetEntityPose(const std::string &_name)
+ignition::math::Pose3d ServerFixture::EntityPose(
+    const std::string &_name)
 {
-  boost::mutex::scoped_lock lock(this->receiveMutex);
+  std::lock_guard<std::mutex> lock(this->receiveMutex);
 
-  std::map<std::string, math::Pose>::iterator iter;
+  std::map<std::string, ignition::math::Pose3d>::iterator iter;
   iter = this->poses.find(_name);
   EXPECT_TRUE(iter != this->poses.end());
   return iter->second;
@@ -371,8 +372,8 @@ math::Pose ServerFixture::GetEntityPose(const std::string &_name)
 /////////////////////////////////////////////////
 bool ServerFixture::HasEntity(const std::string &_name)
 {
-  boost::mutex::scoped_lock lock(this->receiveMutex);
-  std::map<std::string, math::Pose>::iterator iter;
+  std::lock_guard<std::mutex> lock(this->receiveMutex);
+  std::map<std::string, ignition::math::Pose3d>::iterator iter;
   iter = this->poses.find(_name);
   return iter != this->poses.end();
 }
@@ -406,12 +407,12 @@ void ServerFixture::PrintScan(const std::string &_name, double *_scan,
   for (unsigned int i = 0; i < _sampleCount-1; ++i)
   {
     if ((i+1) % 5 == 0)
-      printf("%13.10f,\n", math::precision(_scan[i], 10));
+      printf("%13.10f,\n", ignition::math::precision(_scan[i], 10));
     else
-      printf("%13.10f, ", math::precision(_scan[i], 10));
+      printf("%13.10f, ", ignition::math::precision(_scan[i], 10));
   }
   printf("%13.10f};\n",
-      math::precision(_scan[_sampleCount-1], 10));
+      ignition::math::precision(_scan[_sampleCount-1], 10));
   printf("static double *%s = __%s;\n", _name.c_str(),
       _name.c_str());
 }
@@ -426,8 +427,8 @@ void ServerFixture::FloatCompare(float *_scanA, float *_scanB,
   _diffAvg = 0;
   for (unsigned int i = 0; i < _sampleCount; ++i)
   {
-    double diff = fabs(math::precision(_scanA[i], 10) -
-                math::precision(_scanB[i], 10));
+    double diff = fabs(ignition::math::precision(_scanA[i], 10) -
+                       ignition::math::precision(_scanB[i], 10));
     _diffSum += diff;
     if (diff > _diffMax)
     {
@@ -457,8 +458,8 @@ void ServerFixture::DoubleCompare(double *_scanA, double *_scanB,
     }
     else
     {
-      diff = fabs(math::precision(_scanA[i], 10) -
-                math::precision(_scanB[i], 10));
+      diff = fabs(ignition::math::precision(_scanA[i], 10) -
+                  ignition::math::precision(_scanB[i], 10));
     }
 
     _diffSum += diff;
@@ -538,7 +539,7 @@ void ServerFixture::GetFrame(const std::string &_cameraName,
   while (this->gotImage < 20)
     common::Time::MSleep(100);
 
-  camSensor->Camera()->DisconnectNewImageFrame(c);
+  // c will disconnect automatically when it goes out of scope
 }
 
 /////////////////////////////////////////////////
@@ -565,18 +566,21 @@ physics::ModelPtr ServerFixture::SpawnModel(const msgs::Model &_msg)
     }
   }
 
-  return world->GetModel(_msg.name());
+  return world->ModelByName(_msg.name());
 }
 
 /////////////////////////////////////////////////
 void ServerFixture::SpawnCamera(const std::string &_modelName,
     const std::string &_cameraName,
-    const math::Vector3 &_pos, const math::Vector3 &_rpy,
+    const ignition::math::Vector3d &_pos,
+    const ignition::math::Vector3d &_rpy,
     unsigned int _width, unsigned int _height, double _rate,
     const std::string &_noiseType, double _noiseMean, double _noiseStdDev,
     bool _distortion, double _distortionK1, double _distortionK2,
     double _distortionK3, double _distortionP1, double _distortionP2,
-    double _cx, double _cy)
+    double _cx, double _cy,
+    bool _legacyMode,
+    double _horizontalFov)
 {
   msgs::Factory msg;
   std::ostringstream newModelStr;
@@ -592,7 +596,7 @@ void ServerFixture::SpawnCamera(const std::string &_modelName,
     << "    <update_rate>" << _rate << "</update_rate>"
     << "    <visualize>true</visualize>"
     << "    <camera>"
-    << "      <horizontal_fov>0.78539816339744828</horizontal_fov>"
+    << "      <horizontal_fov>" << _horizontalFov << "</horizontal_fov>"
     << "      <image>"
     << "        <width>" << _width << "</width>"
     << "        <height>" << _height << "</height>"
@@ -604,13 +608,16 @@ void ServerFixture::SpawnCamera(const std::string &_modelName,
     // << "      <save enabled ='true' path ='/tmp/camera/'/>"
 
   if (_noiseType.size() > 0)
+  {
     newModelStr << "      <noise>"
     << "        <type>" << _noiseType << "</type>"
     << "        <mean>" << _noiseMean << "</mean>"
     << "        <stddev>" << _noiseStdDev << "</stddev>"
     << "      </noise>";
+  }
 
   if (_distortion)
+  {
     newModelStr << "      <distortion>"
     << "        <k1>" << _distortionK1 << "</k1>"
     << "        <k2>" << _distortionK2 << "</k2>"
@@ -618,7 +625,11 @@ void ServerFixture::SpawnCamera(const std::string &_modelName,
     << "        <p1>" << _distortionP1 << "</p1>"
     << "        <p2>" << _distortionP2 << "</p2>"
     << "        <center>" << _cx << " " << _cy << "</center>"
+    << "        <ignition:legacy_mode>"
+      << (_legacyMode ? "true" : "false")
+      << "</ignition:legacy_mode>"
     << "      </distortion>";
+  }
 
   newModelStr << "    </camera>"
     << "  </sensor>"
@@ -697,7 +708,7 @@ void ServerFixture::SpawnWideAngleCamera(const std::string &_modelName,
 /////////////////////////////////////////////////
 void ServerFixture::SpawnRaySensor(const std::string &_modelName,
     const std::string &_raySensorName,
-    const math::Vector3 &_pos, const math::Vector3 &_rpy,
+    const ignition::math::Vector3d &_pos, const ignition::math::Vector3d &_rpy,
     double _hMinAngle, double _hMaxAngle,
     double _vMinAngle, double _vMaxAngle,
     double _minRange, double _maxRange,
@@ -747,11 +758,13 @@ void ServerFixture::SpawnRaySensor(const std::string &_modelName,
     << "      </range>";
 
   if (_noiseType.size() > 0)
+  {
     newModelStr << "      <noise>"
     << "        <type>" << _noiseType << "</type>"
     << "        <mean>" << _noiseMean << "</mean>"
     << "        <stddev>" << _noiseStdDev << "</stddev>"
     << "      </noise>";
+  }
 
   newModelStr << "    </ray>"
     << "    <visualize>true</visualize>"
@@ -805,16 +818,34 @@ sensors::SonarSensorPtr ServerFixture::SpawnSonar(const std::string &_modelName,
       sensors::get_sensor(_sonarName));
 }
 
-
 /////////////////////////////////////////////////
 void ServerFixture::SpawnGpuRaySensor(const std::string &_modelName,
     const std::string &_raySensorName,
-    const math::Vector3 &_pos, const math::Vector3 &_rpy,
+    const ignition::math::Vector3d &_pos, const ignition::math::Vector3d &_rpy,
     double _hMinAngle, double _hMaxAngle,
     double _minRange, double _maxRange,
     double _rangeResolution, unsigned int _samples,
     const std::string &_noiseType, double _noiseMean,
     double _noiseStdDev)
+{
+  this->SpawnGpuRaySensorVertical(_modelName, _raySensorName, _pos,
+      _rpy, _hMinAngle, _hMaxAngle, 0, 0, _minRange, _maxRange,
+      _rangeResolution, _samples, 1, 1, 1, _noiseType, _noiseMean,
+      _noiseStdDev);
+}
+
+/////////////////////////////////////////////////
+void ServerFixture::SpawnGpuRaySensorVertical(const std::string &_modelName,
+    const std::string &_raySensorName,
+    const ignition::math::Vector3d &_pos, const ignition::math::Vector3d &_rpy,
+    double _hMinAngle, double _hMaxAngle,
+    double _vMinAngle, double _vMaxAngle,
+    double _minRange, double _maxRange,
+    double _rangeResolution, unsigned int _samples,
+    unsigned int _vSamples, double _hResolution,
+    double _vResolution,
+    const std::string &_noiseType, double _noiseMean,
+    double _noiseStdDev)
 {
   msgs::Factory msg;
   std::ostringstream newModelStr;
@@ -839,10 +870,16 @@ void ServerFixture::SpawnGpuRaySensor(const std::string &_modelName,
     << "      <scan>"
     << "        <horizontal>"
     << "          <samples>" << _samples << "</samples>"
-    << "          <resolution> 1 </resolution>"
+    << "          <resolution>" << _hResolution << "</resolution>"
     << "          <min_angle>" << _hMinAngle << "</min_angle>"
     << "          <max_angle>" << _hMaxAngle << "</max_angle>"
     << "        </horizontal>"
+    << "        <vertical>"
+    << "          <samples>" << _vSamples << "</samples>samples>"
+    << "          <resolution>" << _vResolution << "</resolution>"
+    << "          <min_angle>" << _vMinAngle << "</min_angle>"
+    << "          <max_angle>" << _vMaxAngle << "</max_angle>"
+    << "        </vertical>"
     << "      </scan>"
     << "      <range>"
     << "        <min>" << _minRange << "</min>"
@@ -851,11 +888,13 @@ void ServerFixture::SpawnGpuRaySensor(const std::string &_modelName,
     << "      </range>";
 
   if (_noiseType.size() > 0)
+  {
     newModelStr << "      <noise>"
     << "        <type>" << _noiseType << "</type>"
     << "        <mean>" << _noiseMean << "</mean>"
     << "        <stddev>" << _noiseStdDev << "</stddev>"
     << "      </noise>";
+  }
 
   newModelStr << "    </ray>"
     << "  </sensor>"
@@ -870,10 +909,51 @@ void ServerFixture::SpawnGpuRaySensor(const std::string &_modelName,
   WaitUntilSensorSpawn(_raySensorName, 100, 100);
 }
 
+/////////////////////////////////////////////////
+void ServerFixture::SpawnDepthCameraSensor(const std::string &_modelName,
+    const std::string &_cameraName,
+    const ignition::math::Vector3d &_pos, const ignition::math::Vector3d &_rpy,
+    unsigned int _width, unsigned int _height, double _rate, double _near,
+    double _far)
+{
+  msgs::Factory msg;
+  std::ostringstream newModelStr;
+
+  newModelStr << "<sdf version='" << SDF_VERSION << "'>"
+    << "<model name ='" << _modelName << "'>"
+    << "<static>true</static>"
+    << "<pose>" << _pos << " " << _rpy << "</pose>"
+    << "<link name ='body'>"
+    << "  <sensor name ='" << _cameraName << "' type ='depth'>"
+    << "    <always_on>1</always_on>"
+    << "    <update_rate>" << _rate << "</update_rate>"
+    << "    <visualize>true</visualize>"
+    << "    <camera>"
+    << "      <horizontal_fov>0.78539816339744828</horizontal_fov>"
+    << "      <image>"
+    << "        <width>" << _width << "</width>"
+    << "        <height>" << _height << "</height>"
+    << "      </image>"
+    << "      <clip>"
+    << "        <near>" << _near << "</near><far>" << _far << "</far>"
+    << "      </clip>"
+    << "    </camera>"
+    << "  </sensor>"
+    << "</link>"
+    << "</model>"
+    << "</sdf>";
+
+  msg.set_sdf(newModelStr.str());
+  this->factoryPub->Publish(msg);
+
+  WaitUntilEntitySpawn(_modelName, 100, 50);
+  WaitUntilSensorSpawn(_cameraName, 100, 100);
+}
+
 /////////////////////////////////////////////////
 void ServerFixture::SpawnImuSensor(const std::string &_modelName,
     const std::string &_imuSensorName,
-    const math::Vector3 &_pos, const math::Vector3 &_rpy,
+    const ignition::math::Vector3d &_pos, const ignition::math::Vector3d &_rpy,
     const std::string &_noiseType,
     double _rateNoiseMean, double _rateNoiseStdDev,
     double _rateBiasMean, double _rateBiasStdDev,
@@ -971,8 +1051,9 @@ void ServerFixture::SpawnImuSensor(const std::string &_modelName,
 /////////////////////////////////////////////////
 void ServerFixture::SpawnUnitContactSensor(const std::string &_name,
     const std::string &_sensorName,
-    const std::string &_collisionType, const math::Vector3 &_pos,
-    const math::Vector3 &_rpy, bool _static)
+    const std::string &_collisionType,
+    const ignition::math::Vector3d &_pos, const ignition::math::Vector3d &_rpy,
+    bool _static)
 {
   msgs::Factory msg;
   std::ostringstream newModelStr;
@@ -1031,8 +1112,9 @@ void ServerFixture::SpawnUnitContactSensor(const std::string &_name,
 void ServerFixture::SpawnUnitImuSensor(const std::string &_name,
     const std::string &_sensorName,
     const std::string &_collisionType,
-    const std::string &_topic, const math::Vector3 &_pos,
-    const math::Vector3 &_rpy, bool _static)
+    const std::string &_topic,
+    const ignition::math::Vector3d &_pos, const ignition::math::Vector3d &_rpy,
+    bool _static)
 {
   msgs::Factory msg;
   std::ostringstream newModelStr;
@@ -1201,8 +1283,8 @@ void ServerFixture::launchTimeoutFailure(const char *_logMsg,
 /////////////////////////////////////////////////
 void ServerFixture::SpawnWirelessTransmitterSensor(const std::string &_name,
     const std::string &_sensorName,
-    const math::Vector3 &_pos,
-    const math::Vector3 &_rpy,
+    const ignition::math::Vector3d &_pos,
+    const ignition::math::Vector3d &_rpy,
     const std::string &_essid,
     double _freq,
     double _power,
@@ -1243,8 +1325,8 @@ void ServerFixture::SpawnWirelessTransmitterSensor(const std::string &_name,
 /////////////////////////////////////////////////
 void ServerFixture::SpawnWirelessReceiverSensor(const std::string &_name,
     const std::string &_sensorName,
-    const math::Vector3 &_pos,
-    const math::Vector3 &_rpy,
+    const ignition::math::Vector3d &_pos,
+    const ignition::math::Vector3d &_rpy,
     double _minFreq,
     double _maxFreq,
     double _power,
@@ -1329,7 +1411,7 @@ void ServerFixture::WaitUntilIteration(const uint32_t _goalIteration,
 {
   int i = 0;
   auto world = physics::get_world();
-  while ((world->GetIterations() != _goalIteration) && (i < _retries))
+  while ((world->Iterations() != _goalIteration) && (i < _retries))
   {
     ++i;
     common::Time::MSleep(_sleepEach);
@@ -1342,7 +1424,7 @@ void ServerFixture::WaitUntilSimTime(const common::Time &_goalTime,
 {
   int i = 0;
   auto world = physics::get_world();
-  while ((world->GetSimTime() != _goalTime) && (i < _retries))
+  while ((world->SimTime() != _goalTime) && (i < _retries))
   {
     ++i;
     common::Time::MSleep(_sleepEach);
@@ -1352,18 +1434,19 @@ void ServerFixture::WaitUntilSimTime(const common::Time &_goalTime,
 /////////////////////////////////////////////////
 void ServerFixture::SpawnLight(const std::string &_name,
     const std::string &_type,
-    const math::Vector3 &_pos, const math::Vector3 &_rpy,
-    const common::Color &_diffuse,
-    const common::Color &_specular,
-    const math::Vector3 &_direction,
-    double _attenuationRange,
-    double _attenuationConstant,
-    double _attenuationLinear,
-    double _attenuationQuadratic,
-    double _spotInnerAngle,
-    double _spotOuterAngle,
-    double _spotFallOff,
-    bool _castShadows)
+    const ignition::math::Vector3d &_pos,
+    const ignition::math::Vector3d &_rpy,
+    const ignition::math::Color &_diffuse,
+    const ignition::math::Color &_specular,
+    const ignition::math::Vector3d &_direction,
+    const double _attenuationRange,
+    const double _attenuationConstant,
+    const double _attenuationLinear,
+    const double _attenuationQuadratic,
+    const double _spotInnerAngle,
+    const double _spotOuterAngle,
+    const double _spotFallOff,
+    const bool _castShadows)
 {
   msgs::Factory msg;
   std::ostringstream newLightStr;
@@ -1403,7 +1486,7 @@ void ServerFixture::SpawnLight(const std::string &_name,
   int maxTimeOut = 10;
   while (timeOutCount < maxTimeOut)
   {
-    sceneMsg = world->GetSceneMsg();
+    sceneMsg = world->SceneMsg();
     for (int i = 0; i < sceneMsg.light_size(); ++i)
     {
       if (sceneMsg.light(i).name() == _name)
@@ -1416,7 +1499,7 @@ void ServerFixture::SpawnLight(const std::string &_name,
 
 /////////////////////////////////////////////////
 void ServerFixture::SpawnCylinder(const std::string &_name,
-    const math::Vector3 &_pos, const math::Vector3 &_rpy,
+    const ignition::math::Vector3d &_pos, const ignition::math::Vector3d &_rpy,
     bool _static)
 {
   msgs::Factory msg;
@@ -1424,8 +1507,8 @@ void ServerFixture::SpawnCylinder(const std::string &_name,
   msgs::Model model;
   model.set_name(_name);
   model.set_is_static(_static);
-  msgs::Set(model.mutable_pose(), ignition::math::Pose3d(_pos.Ign(),
-      ignition::math::Quaterniond(_rpy.Ign())));
+  msgs::Set(model.mutable_pose(), ignition::math::Pose3d(_pos,
+      ignition::math::Quaterniond(_rpy)));
   msgs::AddCylinderLink(model, 1.0, 0.5, 1.0);
   auto link = model.mutable_link(0);
   link->set_name("body");
@@ -1445,7 +1528,7 @@ void ServerFixture::SpawnCylinder(const std::string &_name,
 
 /////////////////////////////////////////////////
 void ServerFixture::SpawnSphere(const std::string &_name,
-    const math::Vector3 &_pos, const math::Vector3 &_rpy,
+    const ignition::math::Vector3d &_pos, const ignition::math::Vector3d &_rpy,
     bool _wait, bool _static)
 {
   msgs::Factory msg;
@@ -1480,8 +1563,8 @@ void ServerFixture::SpawnSphere(const std::string &_name,
 
 /////////////////////////////////////////////////
 void ServerFixture::SpawnSphere(const std::string &_name,
-    const math::Vector3 &_pos, const math::Vector3 &_rpy,
-    const math::Vector3 &_cog, double _radius,
+    const ignition::math::Vector3d &_pos, const ignition::math::Vector3d &_rpy,
+    const ignition::math::Vector3d &_cog, double _radius,
     bool _wait, bool _static)
 {
   msgs::Factory msg;
@@ -1489,14 +1572,14 @@ void ServerFixture::SpawnSphere(const std::string &_name,
   msgs::Model model;
   model.set_name(_name);
   model.set_is_static(_static);
-  msgs::Set(model.mutable_pose(), ignition::math::Pose3d(_pos.Ign(),
-      ignition::math::Quaterniond(_rpy.Ign())));
+  msgs::Set(model.mutable_pose(), ignition::math::Pose3d(_pos,
+      ignition::math::Quaterniond(_rpy)));
   msgs::AddSphereLink(model, 1.0, _radius);
   auto link = model.mutable_link(0);
   link->set_name("body");
   link->mutable_collision(0)->set_name("geom");
   msgs::Set(link->mutable_inertial()->mutable_pose(),
-            ignition::math::Pose3d(_cog.Ign(),
+            ignition::math::Pose3d(_cog,
               ignition::math::Quaterniond()));
 
   newModelStr << "<sdf version='" << SDF_VERSION << "'>"
@@ -1513,17 +1596,17 @@ void ServerFixture::SpawnSphere(const std::string &_name,
 
 /////////////////////////////////////////////////
 void ServerFixture::SpawnBox(const std::string &_name,
-    const math::Vector3 &_size, const math::Vector3 &_pos,
-    const math::Vector3 &_rpy, bool _static)
+    const ignition::math::Vector3d &_size, const ignition::math::Vector3d &_pos,
+    const ignition::math::Vector3d &_rpy, bool _static)
 {
   msgs::Factory msg;
   std::ostringstream newModelStr;
   msgs::Model model;
   model.set_name(_name);
   model.set_is_static(_static);
-  msgs::Set(model.mutable_pose(), ignition::math::Pose3d(_pos.Ign(),
-      ignition::math::Quaterniond(_rpy.Ign())));
-  msgs::AddBoxLink(model, 1.0, _size.Ign());
+  msgs::Set(model.mutable_pose(), ignition::math::Pose3d(_pos,
+      ignition::math::Quaterniond(_rpy)));
+  msgs::AddBoxLink(model, 1.0, _size);
   auto link = model.mutable_link(0);
   link->set_name("body");
   link->mutable_collision(0)->set_name("geom");
@@ -1542,8 +1625,8 @@ void ServerFixture::SpawnBox(const std::string &_name,
 
 /////////////////////////////////////////////////
 void ServerFixture::SpawnTrimesh(const std::string &_name,
-    const std::string &_modelPath, const math::Vector3 &_scale,
-    const math::Vector3 &_pos, const math::Vector3 &_rpy,
+    const std::string &_modelPath, const ignition::math::Vector3d &_scale,
+    const ignition::math::Vector3d &_pos, const ignition::math::Vector3d &_rpy,
     bool _static)
 {
   msgs::Factory msg;
@@ -1581,7 +1664,7 @@ void ServerFixture::SpawnTrimesh(const std::string &_name,
 
 /////////////////////////////////////////////////
 void ServerFixture::SpawnEmptyLink(const std::string &_name,
-    const math::Vector3 &_pos, const math::Vector3 &_rpy,
+    const ignition::math::Vector3d &_pos, const ignition::math::Vector3d &_rpy,
     bool _static)
 {
   msgs::Factory msg;
@@ -1589,8 +1672,8 @@ void ServerFixture::SpawnEmptyLink(const std::string &_name,
   msgs::Model model;
   model.set_name(_name);
   model.set_is_static(_static);
-  msgs::Set(model.mutable_pose(), ignition::math::Pose3d(_pos.Ign(),
-      ignition::math::Quaterniond(_rpy.Ign())));
+  msgs::Set(model.mutable_pose(), ignition::math::Pose3d(_pos,
+      ignition::math::Quaterniond(_rpy)));
   model.add_link();
   model.mutable_link(0)->set_name("body");
 
@@ -1656,7 +1739,7 @@ physics::ModelPtr ServerFixture::GetModel(const std::string &_name)
 {
   // Get the first world...we assume it the only one running
   physics::WorldPtr world = physics::get_world();
-  return world->GetModel(_name);
+  return world->ModelByName(_name);
 }
 
 /////////////////////////////////////////////////
