@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2016 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -41,6 +41,7 @@ class ContactSensor : public ServerFixture,
   public: void MultipleSensors(const std::string &_physicsEngine);
   public: void StackTest(const std::string &_physicsEngine);
   public: void TorqueTest(const std::string &_physicsEngine);
+  public: void StrictUpdateRateTest(const std::string &_physicsEngine);
 
   /// \brief Callback for sensor subscribers in MultipleSensors test.
   private: void Callback(const ConstContactsPtr &_msg);
@@ -64,7 +65,7 @@ void ContactSensor::ModelRemoval(const std::string &_physicsEngine)
   ASSERT_TRUE(world != nullptr);
 
   // Verify physics engine type
-  physics::PhysicsEnginePtr physics = world->GetPhysicsEngine();
+  physics::PhysicsEnginePtr physics = world->Physics();
   ASSERT_TRUE(physics != nullptr);
   EXPECT_EQ(physics->GetType(), _physicsEngine);
 
@@ -106,7 +107,7 @@ void ContactSensor::ModelRemoval(const std::string &_physicsEngine)
 
   EXPECT_TRUE(contactSensor->IsActive());
 
-  physics::ModelPtr contactModel = world->GetModel(modelName);
+  physics::ModelPtr contactModel = world->ModelByName(modelName);
   ASSERT_TRUE(contactModel != nullptr);
 
   // check new topic are published
@@ -138,7 +139,7 @@ void ContactSensor::ModelRemoval(const std::string &_physicsEngine)
   // remove the model
   world->RemoveModel(contactModel);
 
-  contactModel = world->GetModel(modelName);
+  contactModel = world->ModelByName(modelName);
   EXPECT_TRUE(contactModel == nullptr);
 
   int sleep = 0;
@@ -189,8 +190,8 @@ void ContactSensor::MoveTool(const std::string &_physicsEngine)
   ASSERT_TRUE(world != NULL);
 
   const std::string modelName("sphere");
-  const math::Vector3 pos(0, 0, 1.8);
-  const math::Vector3 v30;
+  const ignition::math::Vector3d pos(0, 0, 1.8);
+  const ignition::math::Vector3d v30;
   const double radius = 0.5;
   SpawnSphere(modelName, pos, v30, v30, radius);
 
@@ -203,20 +204,20 @@ void ContactSensor::MoveTool(const std::string &_physicsEngine)
   world->Step(200);
 
   // Try moving the model
-  auto model = world->GetModel(modelName);
+  auto model = world->ModelByName(modelName);
   ASSERT_TRUE(model != NULL);
 
-  auto pose = model->GetWorldPose();
-  pose.pos.x += 0.2;
-  pose.pos.y += 0.2;
+  auto pose = model->WorldPose();
+  pose.Pos().X() += 0.2;
+  pose.Pos().Y() += 0.2;
 
   msgs::Model msg;
   msg.set_name(modelName);
   msg.set_id(model->GetId());
-  msgs::Set(msg.mutable_pose(), pose.Ign());
+  msgs::Set(msg.mutable_pose(), pose);
   modelPub->Publish(msg);
 
-  while (pose != model->GetWorldPose())
+  while (pose != model->WorldPose())
   {
     world->Step(1);
     common::Time::MSleep(1);
@@ -234,7 +235,7 @@ TEST_P(ContactSensor, MoveTool)
 
 ////////////////////////////////////////////////////////////////////////
 // Test having multiple contact sensors under a single link.
-// https://bitbucket.org/osrf/gazebo/issue/960
+// https://github.com/osrf/gazebo/issues/960
 ////////////////////////////////////////////////////////////////////////
 void ContactSensor::MultipleSensors(const std::string &_physicsEngine)
 {
@@ -320,33 +321,33 @@ void ContactSensor::StackTest(const std::string &_physicsEngine)
   ASSERT_TRUE(world != NULL);
 
   // Verify physics engine type
-  physics::PhysicsEnginePtr physics = world->GetPhysicsEngine();
+  physics::PhysicsEnginePtr physics = world->Physics();
   ASSERT_TRUE(physics != NULL);
   EXPECT_EQ(physics->GetType(), _physicsEngine);
 
   std::string modelName01 = "contactModel01";
   std::string contactSensorName01 = "contactSensor01";
-  math::Pose modelPose01(0, 0, 0.5, 0, 0, 0);
+  ignition::math::Pose3d modelPose01(0, 0, 0.5, 0, 0, 0);
 
   std::string modelName02 = "contactModel02";
   std::string contactSensorName02 = "contactSensor02";
-  math::Pose modelPose02(0, 2, 0.5, 0, M_PI/2.0, 0);
+  ignition::math::Pose3d modelPose02(0, 2, 0.5, 0, M_PI/2.0, 0);
 
   std::string sphereName01 = "sphere01";
-  math::Pose spherePose01(0, 0, 1.5, 0, 0, 0);
+  ignition::math::Pose3d spherePose01(0, 0, 1.5, 0, 0, 0);
 
   std::string sphereName02 = "sphere02";
-  math::Pose spherePose02(0, 2, 1.5, 0, 0, 0);
+  ignition::math::Pose3d spherePose02(0, 2, 1.5, 0, 0, 0);
 
   // spawn two contact sensors
   SpawnUnitContactSensor(modelName01, contactSensorName01,
-      "box", modelPose01.pos, modelPose01.rot.GetAsEuler());
+      "box", modelPose01.Pos(), modelPose01.Rot().Euler());
   SpawnUnitContactSensor(modelName02, contactSensorName02,
-      "box", modelPose02.pos, modelPose02.rot.GetAsEuler());
+      "box", modelPose02.Pos(), modelPose02.Rot().Euler());
 
   // spawn two spheres, each sphere rests on top of one contact sensor
-  SpawnSphere(sphereName01, spherePose01.pos, spherePose01.rot.GetAsEuler());
-  SpawnSphere(sphereName02, spherePose02.pos, spherePose02.rot.GetAsEuler());
+  SpawnSphere(sphereName01, spherePose01.Pos(), spherePose01.Rot().Euler());
+  SpawnSphere(sphereName02, spherePose02.Pos(), spherePose02.Rot().Euler());
 
   sensors::SensorPtr sensor01 = sensors::get_sensor(contactSensorName01);
   sensors::ContactSensorPtr contactSensor01 =
@@ -377,8 +378,8 @@ void ContactSensor::StackTest(const std::string &_physicsEngine)
   EXPECT_TRUE(contactSensor01->IsActive());
   EXPECT_TRUE(contactSensor02->IsActive());
 
-  physics::ModelPtr contactModel01 = world->GetModel(modelName01);
-  physics::ModelPtr contactModel02 = world->GetModel(modelName02);
+  physics::ModelPtr contactModel01 = world->ModelByName(modelName01);
+  physics::ModelPtr contactModel02 = world->ModelByName(modelName02);
   ASSERT_TRUE(contactModel01 != NULL);
   ASSERT_TRUE(contactModel02 != NULL);
 
@@ -387,7 +388,7 @@ void ContactSensor::StackTest(const std::string &_physicsEngine)
   models.push_back(contactModel02);
 
   double gravityZ = -9.8;
-  physics->SetGravity(math::Vector3(0, 0, gravityZ));
+  physics->SetGravity(ignition::math::Vector3d(0, 0, gravityZ));
 
   msgs::Contacts contacts01;
   msgs::Contacts contacts02;
@@ -413,8 +414,8 @@ void ContactSensor::StackTest(const std::string &_physicsEngine)
   contacts.push_back(contacts01);
   contacts.push_back(contacts02);
 
-  math::Vector3 expectedForce;
-  math::Vector3 expectedTorque;
+  ignition::math::Vector3d expectedForce;
+  ignition::math::Vector3d expectedTorque;
 
   // double tolPercentage = 0.1;
   // double tol = 1e-2;
@@ -423,10 +424,10 @@ void ContactSensor::StackTest(const std::string &_physicsEngine)
   // Run the test once for each contact sensor
   for (unsigned int k = 0; k < contacts.size(); ++k)
   {
-    double mass = models[k]->GetLink()->GetInertial()->GetMass();
-    expectedForce = models[k]->GetLink()->GetWorldCoGPose().rot.GetInverse()
-        * math::Vector3(0, 0, (gravityZ * mass));
-    expectedTorque = math::Vector3(0, 0, 0);
+    double mass = models[k]->GetLink()->GetInertial()->Mass();
+    expectedForce = models[k]->GetLink()->WorldCoGPose().Rot().Inverse()
+        * ignition::math::Vector3d(0, 0, (gravityZ * mass));
+    expectedTorque = ignition::math::Vector3d::Zero;
 
     unsigned int ColInd = 0;
     physics::CollisionPtr col = models[k]->GetLink()->GetCollision(ColInd);
@@ -435,9 +436,9 @@ void ContactSensor::StackTest(const std::string &_physicsEngine)
     // calculate tolerance based on magnitude of force
     // Uncomment lines below once we are able to accurately determine the
     // expected force output, see issue #565
-    // tolX = std::max(tolPercentage*expectedForce.x, tol);
-    // tolY = std::max(tolPercentage*expectedForce.y, tol);
-    // tolZ = std::max(tolPercentage*expectedForce.z, tol);
+    // tolX = std::max(tolPercentage*expectedForce.X(), tol);
+    // tolY = std::max(tolPercentage*expectedForce.Y(), tol);
+    // tolZ = std::max(tolPercentage*expectedForce.Z(), tol);
 
     // loop through contact collision pairs
     for (int i = 0; i < contacts[k].contact_size(); ++i)
@@ -452,21 +453,21 @@ void ContactSensor::StackTest(const std::string &_physicsEngine)
         FAIL();
       }
 
-      math::Vector3 actualForce;
-      math::Vector3 actualTorque;
+      ignition::math::Vector3d actualForce;
+      ignition::math::Vector3d actualTorque;
 
       // loop through all contact points between the two collisions
       for (int j = 0; j < contacts[k].contact(i).position_size(); ++j)
       {
         // Contact between the sphere and the contact sensor occurs at z=1.0
         // Skip other contact points with the ground plane
-        if (!math::equal(contacts[k].contact(i).position(j).z(), 1.0))
+        if (!ignition::math::equal(contacts[k].contact(i).position(j).z(), 1.0))
           continue;
 
         EXPECT_NEAR(contacts[k].contact(i).position(j).x(),
-            models[k]->GetLink()->GetWorldCoGPose().pos.x, TOL);
+            models[k]->GetLink()->WorldCoGPose().Pos().X(), TOL);
         EXPECT_NEAR(contacts[k].contact(i).position(j).y(),
-            models[k]->GetLink()->GetWorldCoGPose().pos.y, TOL);
+            models[k]->GetLink()->WorldCoGPose().Pos().Y(), TOL);
 
         EXPECT_NEAR(contacts[k].contact(i).normal(j).x(), 0, TOL);
         EXPECT_NEAR(contacts[k].contact(i).normal(j).y(), 0, TOL);
@@ -474,51 +475,51 @@ void ContactSensor::StackTest(const std::string &_physicsEngine)
 
         if (body1)
         {
-          actualForce.x =
+          actualForce.X() =
             contacts[k].contact(i).wrench(j).body_1_wrench().force().x();
-          actualForce.y =
+          actualForce.Y() =
             contacts[k].contact(i).wrench(j).body_1_wrench().force().y();
-          actualForce.z =
+          actualForce.Z() =
             contacts[k].contact(i).wrench(j).body_1_wrench().force().z();
 
-          actualTorque.x =
+          actualTorque.X() =
             contacts[k].contact(i).wrench(j).body_1_wrench().torque().x();
-          actualTorque.y =
+          actualTorque.Y() =
             contacts[k].contact(i).wrench(j).body_1_wrench().torque().y();
-          actualTorque.z =
+          actualTorque.Z() =
             contacts[k].contact(i).wrench(j).body_1_wrench().torque().z();
         }
         else
         {
-          actualForce.x =
+          actualForce.X() =
             contacts[k].contact(i).wrench(j).body_2_wrench().force().x();
-          actualForce.y =
+          actualForce.Y() =
             contacts[k].contact(i).wrench(j).body_2_wrench().force().y();
-          actualForce.z =
+          actualForce.Z() =
             contacts[k].contact(i).wrench(j).body_2_wrench().force().z();
 
-          actualTorque.x =
+          actualTorque.X() =
             contacts[k].contact(i).wrench(j).body_2_wrench().torque().x();
-          actualTorque.y =
+          actualTorque.Y() =
             contacts[k].contact(i).wrench(j).body_2_wrench().torque().y();
-          actualTorque.z =
+          actualTorque.Z() =
             contacts[k].contact(i).wrench(j).body_2_wrench().torque().z();
         }
 
         // Find the dominant force vector component and verify the value has
         // the correct sign.
         // Force and torque are given in the link frame, see issue #545
-        int vi = (fabs(expectedForce.x) > fabs(expectedForce.y))
+        int vi = (fabs(expectedForce.X()) > fabs(expectedForce.Y()))
             ? 0 : 1;
-        vi = (fabs(expectedForce[vi]) > fabs(expectedForce.z))
+        vi = (fabs(expectedForce[vi]) > fabs(expectedForce.Z()))
             ? vi : 2;
         EXPECT_EQ((expectedForce[vi] < 0), (actualForce[vi] < 0));
 
         // Verify torque with a large tolerance
         double odeTorqueTol = 2;
-        EXPECT_LT(fabs(actualTorque.x), odeTorqueTol);
-        EXPECT_LT(fabs(actualTorque.y), odeTorqueTol);
-        EXPECT_LT(fabs(actualTorque.z), odeTorqueTol);
+        EXPECT_LT(fabs(actualTorque.X()), odeTorqueTol);
+        EXPECT_LT(fabs(actualTorque.Y()), odeTorqueTol);
+        EXPECT_LT(fabs(actualTorque.Z()), odeTorqueTol);
 
         // TODO: ODE and bullet produce slightly different results
         // In ODE the force and torque on an object seem to be affected by other
@@ -526,12 +527,12 @@ void ContactSensor::StackTest(const std::string &_physicsEngine)
         // exerts non-zero x and y forces on the contact sensor when only
         // negative z forces are expected.
         // The tests below pass in bullet but fail in ode, see issue #565
-        // EXPECT_NEAR(expectedForce.x, actualForce.x, tolX);
-        // EXPECT_NEAR(expectedForce.y, actualForce.y, tolY);
-        // EXPECT_NEAR(expectedForce.z, actualForce.z, tolZ);
-        // EXPECT_NEAR(expectedTorque.x, actualTorque.x, tolX);
-        // EXPECT_NEAR(expectedTorque.y, actualTorque.y, tolY);
-        // EXPECT_NEAR(expectedTorque.z, actualTorque.z, tolZ);
+        // EXPECT_NEAR(expectedForce.X(), actualForce.X(), tolX);
+        // EXPECT_NEAR(expectedForce.Y(), actualForce.Y(), tolY);
+        // EXPECT_NEAR(expectedForce.Z(), actualForce.Z(), tolZ);
+        // EXPECT_NEAR(expectedTorque.X(), actualTorque.X(), tolX);
+        // EXPECT_NEAR(expectedTorque.Y(), actualTorque.Y(), tolY);
+        // EXPECT_NEAR(expectedTorque.Z(), actualTorque.Z(), tolZ);
       }
     }
   }
@@ -556,21 +557,21 @@ void ContactSensor::TorqueTest(const std::string &_physicsEngine)
   ASSERT_TRUE(world != NULL);
 
   // Verify physics engine type
-  physics::PhysicsEnginePtr physics = world->GetPhysicsEngine();
+  physics::PhysicsEnginePtr physics = world->Physics();
   ASSERT_TRUE(physics != NULL);
   EXPECT_EQ(physics->GetType(), _physicsEngine);
 
   std::string modelName = "contactModel";
   std::string contactSensorName = "contactSensor";
-  math::Pose modelPose(0, -0.3, 1.5, M_PI/2.0, 0, 0);
+  ignition::math::Pose3d modelPose(0, -0.3, 1.5, M_PI/2.0, 0, 0);
 
   std::string cylinderName = "cylinder";
-  math::Pose cylinderPose(0, 0, 0.5, 0, M_PI/2.0, 0);
+  ignition::math::Pose3d cylinderPose(0, 0, 0.5, 0, M_PI/2.0, 0);
 
   SpawnUnitContactSensor(modelName, contactSensorName,
-      "cylinder", modelPose.pos, modelPose.rot.GetAsEuler());
+      "cylinder", modelPose.Pos(), modelPose.Rot().Euler());
 
-  SpawnCylinder(cylinderName, cylinderPose.pos, cylinderPose.rot.GetAsEuler());
+  SpawnCylinder(cylinderName, cylinderPose.Pos(), cylinderPose.Rot().Euler());
 
   sensors::SensorPtr sensor = sensors::get_sensor(contactSensorName);
   sensors::ContactSensorPtr contactSensor =
@@ -590,11 +591,11 @@ void ContactSensor::TorqueTest(const std::string &_physicsEngine)
 
   EXPECT_TRUE(contactSensor->IsActive());
 
-  physics::ModelPtr contactModel = world->GetModel(modelName);
+  physics::ModelPtr contactModel = world->ModelByName(modelName);
   ASSERT_TRUE(contactModel != NULL);
 
   double gravityZ = -9.8;
-  physics->SetGravity(math::Vector3(0, 0, gravityZ));
+  physics->SetGravity(ignition::math::Vector3d(0, 0, gravityZ));
 
   msgs::Contacts contacts;
 
@@ -602,7 +603,9 @@ void ContactSensor::TorqueTest(const std::string &_physicsEngine)
   {
     EXPECT_TRUE(physics->SetParam("iters", 100));
     if (_physicsEngine == "ode")
+    {
       EXPECT_TRUE(physics->SetParam("contact_max_correcting_vel", 0));
+    }
   }
 
   world->Step(1);
@@ -636,7 +639,7 @@ void ContactSensor::TorqueTest(const std::string &_physicsEngine)
     {
       FAIL();
     }
-    math::Vector3 actualTorque;
+    ignition::math::Vector3d actualTorque;
 
     // loop through all contact points between the two collisions
     for (int j = 0; j < contacts.contact(i).position_size(); ++j)
@@ -646,20 +649,20 @@ void ContactSensor::TorqueTest(const std::string &_physicsEngine)
 
       if (body1)
       {
-        actualTorque.x =
+        actualTorque.X() =
           contacts.contact(i).wrench(j).body_1_wrench().torque().x();
-        actualTorque.y =
+        actualTorque.Y() =
           contacts.contact(i).wrench(j).body_1_wrench().torque().y();
-        actualTorque.z =
+        actualTorque.Z() =
           contacts.contact(i).wrench(j).body_1_wrench().torque().z();
       }
       else
       {
-        actualTorque.x =
+        actualTorque.X() =
           contacts.contact(i).wrench(j).body_2_wrench().torque().x();
-        actualTorque.y =
+        actualTorque.Y() =
           contacts.contact(i).wrench(j).body_2_wrench().torque().y();
-        actualTorque.z =
+        actualTorque.Z() =
           contacts.contact(i).wrench(j).body_2_wrench().torque().z();
       }
 
@@ -668,11 +671,11 @@ void ContactSensor::TorqueTest(const std::string &_physicsEngine)
       {
         // contact sensor should have positive x torque and relatively large
         // compared to y and z
-        EXPECT_GT(actualTorque.x, 0);
-        EXPECT_GT(actualTorque.x, fabs(actualTorque.y));
-        EXPECT_GT(actualTorque.x, fabs(actualTorque.z));
-        // EXPECT_LT(fabs(actualTorque.y), tol);
-        // EXPECT_LT(fabs(actualTorque.z), tol);
+        EXPECT_GT(actualTorque.X(), 0);
+        EXPECT_GT(actualTorque.X(), fabs(actualTorque.Y()));
+        EXPECT_GT(actualTorque.X(), fabs(actualTorque.Z()));
+        // EXPECT_LT(fabs(actualTorque.Y()), tol);
+        // EXPECT_LT(fabs(actualTorque.Z()), tol);
       }
     }
   }
@@ -683,7 +686,64 @@ TEST_P(ContactSensor, TorqueTest)
   TorqueTest(GetParam());
 }
 
-INSTANTIATE_TEST_CASE_P(PhysicsEngines, ContactSensor, PHYSICS_ENGINE_VALUES);
+////////////////////////////////////////////////////////////////////////
+// Test strict update rate of contact sensor.
+////////////////////////////////////////////////////////////////////////
+void ContactSensor::StrictUpdateRateTest(const std::string &_physicsEngine)
+{
+  LoadArgs(" --lockstep -u -e " + _physicsEngine +
+      " worlds/contact_strict_rate.world");
+
+  // Wait until the sensors have been initialized
+  while (!sensors::SensorManager::Instance()->SensorsInitialized())
+    common::Time::MSleep(1000);
+
+  std::string contactSensorName = "sensor_contact";
+  sensors::SensorPtr sensor = sensors::get_sensor(contactSensorName);
+  sensors::ContactSensorPtr contactSensor =
+      std::dynamic_pointer_cast<sensors::ContactSensor>(sensor);
+  ASSERT_TRUE(contactSensor != NULL);
+
+  g_messageCount = 0;
+  transport::SubscriberPtr sub = this->node->Subscribe(
+      "/gazebo/world_1/physics/contacts", &ContactSensor::Callback, this);
+  common::Timer timer;
+  SetPause(false);
+  timer.Start();
+
+  // how many msgs produced for 5 seconds (in simulated clock domain)
+  double updateRate = contactSensor->UpdateRate();
+  unsigned int totalMsgs = 5 * updateRate;
+  physics::WorldPtr world = physics::get_world("world_1");
+  ASSERT_TRUE(world != NULL);
+  double simT0 = 0.0;
+
+  while (g_messageCount < totalMsgs)
+  {
+    // An approximation of when we receive the first image. In reality one
+    // iteration before we receive the second image.
+    if (g_messageCount == 0)
+    {
+      simT0 = world->SimTime().Double();
+    }
+    common::Time::MSleep(1);
+  }
+
+  // check that the obtained rate is the one expected
+  double dt = world->SimTime().Double() - simT0;
+  double rate = static_cast<double>(totalMsgs) / dt;
+  gzdbg << "timer [" << dt << "] seconds rate [" << rate << "] fps\n";
+  const double tolerance = 0.02;
+  EXPECT_GT(rate, updateRate * (1 - tolerance));
+  EXPECT_LT(rate, updateRate * (1 + tolerance));
+}
+
+TEST_P(ContactSensor, StrictUpdateRateTest)
+{
+  StrictUpdateRateTest(GetParam());
+}
+
+INSTANTIATE_TEST_CASE_P(PhysicsEngines, ContactSensor, PHYSICS_ENGINE_VALUES,);  // NOLINT
 
 int main(int argc, char **argv)
 {
