@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Open Source Robotics Foundation
+ * Copyright (C) 2012 Open Source Robotics Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -41,8 +41,8 @@ void PR2Test::LoadPR2(std::string _physicsEngine)
 
   // Cleanup test directory.
   common::SystemPaths *paths = common::SystemPaths::Instance();
-  boost::filesystem::remove_all(paths->GetDefaultTestPath());
-  boost::filesystem::create_directories(paths->GetDefaultTestPath());
+  boost::filesystem::remove_all(paths->DefaultTestPath());
+  boost::filesystem::create_directories(paths->DefaultTestPath());
 
   ServerFixture::Load("worlds/empty.world", false, _physicsEngine);
   SpawnModel("model://pr2");
@@ -62,24 +62,24 @@ void PR2Test::LoadPR2(std::string _physicsEngine)
   EXPECT_TRUE(sensor != NULL);
 
   sensors::CameraSensorPtr camSensor =
-    boost::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
+    std::dynamic_pointer_cast<sensors::CameraSensor>(sensor);
   EXPECT_TRUE(camSensor != NULL);
 
-  while (!camSensor->SaveFrame(paths->GetDefaultTestPath() + "/frame_10.jpg"))
+  while (!camSensor->SaveFrame(paths->DefaultTestPath() + "/frame_10.jpg"))
     common::Time::MSleep(100);
 
-  physics::get_world("default")->GetPhysicsEngine()->SetGravity(
-      math::Vector3(-0.5, 0, -0.1));
+  physics::get_world("default")->Physics()->SetGravity(
+      ignition::math::Vector3d(-0.5, 0, -0.1));
   for (int i = 11; i < 200; i++)
   {
     std::ostringstream filename;
-    filename << paths->GetDefaultTestPath() << "/frame_" << i << ".jpg";
+    filename << paths->DefaultTestPath() << "/frame_" << i << ".jpg";
     camSensor->SaveFrame(filename.str());
     common::Time::MSleep(100);
   }
 
   // Cleanup test directory.
-  boost::filesystem::remove_all(paths->GetDefaultTestPath());
+  boost::filesystem::remove_all(paths->DefaultTestPath());
 }
 
 TEST_P(PR2Test, LoadPR2)
@@ -107,11 +107,11 @@ void PR2Test::ScrewJoint(std::string _physicsEngine)
   ASSERT_TRUE(world != NULL);
 
   // check the physics engine type
-  physics::PhysicsEnginePtr physics = world->GetPhysicsEngine();
+  physics::PhysicsEnginePtr physics = world->Physics();
   ASSERT_TRUE(physics != NULL);
   EXPECT_EQ(physics->GetType(), _physicsEngine);
 
-  physics::ModelPtr model = world->GetModel("pr2");
+  physics::ModelPtr model = world->ModelByName("pr2");
   ASSERT_TRUE(model != NULL);
 
   physics::LinkPtr link = model->GetLink("torso_lift_link");
@@ -139,8 +139,8 @@ void PR2Test::ScrewJoint(std::string _physicsEngine)
   // Expect torso to lift at least 1 mm/s
   gzdbg << "motor " << motor->GetVelocity(0) << std::endl;
   gzdbg << "screw " << screw->GetVelocity(0) << std::endl;
-  gzdbg << "link  " << link->GetWorldLinearVel() << std::endl;
-  EXPECT_GT(link->GetWorldLinearVel().z, 1e-3);
+  gzdbg << "link  " << link->WorldLinearVel() << std::endl;
+  EXPECT_GT(link->WorldLinearVel().Z(), 1e-3);
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -173,22 +173,22 @@ void PR2Test::StaticPR2(std::string _physicsEngine)
   ASSERT_TRUE(world != NULL);
 
   // Verify physics engine type
-  physics::PhysicsEnginePtr physics = world->GetPhysicsEngine();
+  physics::PhysicsEnginePtr physics = world->Physics();
   ASSERT_TRUE(physics != NULL);
   EXPECT_EQ(physics->GetType(), _physicsEngine);
 
   // simulate 1 step
   world->Step(1);
-  double t = world->GetSimTime().Double();
+  double t = world->SimTime().Double();
   // verify that time moves forward
   EXPECT_GT(t, 0);
 
   // simulate a few steps
   int steps = 20;
   world->Step(steps);
-  double dt = world->GetPhysicsEngine()->GetMaxStepSize();
+  double dt = world->Physics()->GetMaxStepSize();
   EXPECT_GT(dt, 0);
-  t = world->GetSimTime().Double();
+  t = world->SimTime().Double();
   EXPECT_GT(t, 0.99*dt*static_cast<double>(steps+1));
 }
 
@@ -202,7 +202,7 @@ TEST_P(PR2Test, StaticPR2)
   StaticPR2(GetParam());
 }
 
-INSTANTIATE_TEST_CASE_P(PhysicsEngines, PR2Test, PHYSICS_ENGINE_VALUES);
+INSTANTIATE_TEST_CASE_P(PhysicsEngines, PR2Test, PHYSICS_ENGINE_VALUES,);  // NOLINT
 
 int main(int argc, char **argv)
 {
